@@ -104,15 +104,17 @@ func WriteToDisk(filename string, kubeconfig *clientcmdapi.Config) error {
 }
 
 // GetClusterFromKubeConfig returns the default Cluster of the specified KubeConfig
-func GetClusterFromKubeConfig(config *clientcmdapi.Config) *clientcmdapi.Cluster {
+func GetClusterFromKubeConfig(config *clientcmdapi.Config) (string, *clientcmdapi.Cluster) {
 	// If there is an unnamed cluster object, use it
 	if config.Clusters[""] != nil {
-		return config.Clusters[""]
+		return "", config.Clusters[""]
 	}
-	if config.Contexts[config.CurrentContext] != nil {
-		return config.Clusters[config.Contexts[config.CurrentContext].Cluster]
+
+	currentContext := config.Contexts[config.CurrentContext]
+	if currentContext != nil {
+		return currentContext.Cluster, config.Clusters[currentContext.Cluster]
 	}
-	return nil
+	return "", nil
 }
 
 // HasAuthenticationCredentials returns true if the current user has valid authentication credentials for
@@ -124,7 +126,7 @@ func HasAuthenticationCredentials(config *clientcmdapi.Config) bool {
 	}
 
 	// token authentication
-	if len(authInfo.Token) != 0 {
+	if len(authInfo.Token) != 0 || len(authInfo.TokenFile) != 0 {
 		return true
 	}
 
@@ -139,6 +141,16 @@ func HasAuthenticationCredentials(config *clientcmdapi.Config) bool {
 		return true
 	}
 
+	// exec authentication
+	if authInfo.Exec != nil && len(authInfo.Exec.Command) != 0 {
+		return true
+	}
+
+	// authprovider authentication
+	if authInfo.AuthProvider != nil && len(authInfo.AuthProvider.Name) != 0 {
+		return true
+	}
+
 	return false
 }
 
@@ -166,6 +178,14 @@ func EnsureAuthenticationInfoAreEmbedded(config *clientcmdapi.Config) error {
 		authInfo.ClientKeyData = clientKey
 		authInfo.ClientKey = ""
 	}
+	if len(authInfo.Token) == 0 && len(authInfo.TokenFile) != 0 {
+		tokenBytes, err := os.ReadFile(authInfo.TokenFile)
+		if err != nil {
+			return errors.Wrap(err, "error while reading token file defined in kubeconfig")
+		}
+		authInfo.Token = string(tokenBytes)
+		authInfo.TokenFile = ""
+	}
 
 	return nil
 }
