@@ -18,9 +18,7 @@ package cronjob
 
 import (
 	"context"
-	"fmt"
 	"reflect"
-	"sort"
 	"strings"
 	"testing"
 	"time"
@@ -32,19 +30,14 @@ import (
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/apiserver/pkg/util/feature"
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/kubernetes/fake"
 	"k8s.io/client-go/tools/record"
 	"k8s.io/client-go/util/workqueue"
-	featuregatetesting "k8s.io/component-base/featuregate/testing"
 	"k8s.io/klog/v2/ktesting"
-	"k8s.io/utils/pointer"
-
 	_ "k8s.io/kubernetes/pkg/apis/batch/install"
 	_ "k8s.io/kubernetes/pkg/apis/core/install"
 	"k8s.io/kubernetes/pkg/controller"
-	"k8s.io/kubernetes/pkg/features"
 )
 
 var (
@@ -207,7 +200,6 @@ func TestControllerV2SyncCronJob(t *testing.T) {
 		now                 time.Time
 		jobCreateError      error
 		jobGetErr           error
-		enableTimeZone      bool
 
 		// expectations
 		expectCreate               bool
@@ -255,7 +247,6 @@ func TestControllerV2SyncCronJob(t *testing.T) {
 			deadline:                   noDead,
 			jobCreationTime:            justAfterThePriorHour(),
 			now:                        justBeforeTheHour(),
-			enableTimeZone:             true,
 			expectedWarnings:           1,
 			jobPresentInCJActiveStatus: true,
 		},
@@ -295,7 +286,6 @@ func TestControllerV2SyncCronJob(t *testing.T) {
 			deadline:                   noDead,
 			jobCreationTime:            justAfterThePriorHour(),
 			now:                        justBeforeTheHour(),
-			enableTimeZone:             true,
 			expectRequeueAfter:         true,
 			expectedRequeueDuration:    1*time.Minute + nextScheduleDelta,
 			jobPresentInCJActiveStatus: true,
@@ -346,7 +336,6 @@ func TestControllerV2SyncCronJob(t *testing.T) {
 			deadline:                   noDead,
 			jobCreationTime:            justAfterThePriorHour(),
 			now:                        justAfterTheHourInZone(newYork),
-			enableTimeZone:             false,
 			expectCreate:               true,
 			expectActive:               1,
 			expectRequeueAfter:         true,
@@ -361,7 +350,6 @@ func TestControllerV2SyncCronJob(t *testing.T) {
 			deadline:                   noDead,
 			jobCreationTime:            justAfterThePriorHour(),
 			now:                        justAfterTheHourInZone(newYork),
-			enableTimeZone:             true,
 			expectCreate:               true,
 			expectActive:               1,
 			expectRequeueAfter:         true,
@@ -376,7 +364,6 @@ func TestControllerV2SyncCronJob(t *testing.T) {
 			deadline:                   noDead,
 			jobCreationTime:            justAfterThePriorHour(),
 			now:                        justAfterTheHourInZone(newYork),
-			enableTimeZone:             true,
 			expectCreate:               true,
 			expectedWarnings:           1,
 			expectRequeueAfter:         true,
@@ -475,22 +462,10 @@ func TestControllerV2SyncCronJob(t *testing.T) {
 			jobCreationTime:            justAfterThePriorHour(),
 			now:                        *justAfterTheHour(),
 			jobCreateError:             errors.NewAlreadyExists(schema.GroupResource{Resource: "job", Group: "batch"}, ""),
-			expectErr:                  false,
+			expectErr:                  true,
 			expectUpdateStatus:         true,
 			jobPresentInCJActiveStatus: true,
 		},
-		"prev ran but done, is time, job not present in CJ active status, create job failed, A": {
-			concurrencyPolicy:          "Allow",
-			schedule:                   onTheHour,
-			deadline:                   noDead,
-			ranPreviously:              true,
-			jobCreationTime:            justAfterThePriorHour(),
-			now:                        *justAfterTheHour(),
-			jobCreateError:             errors.NewAlreadyExists(schema.GroupResource{Resource: "job", Group: "batch"}, ""),
-			expectErr:                  false,
-			expectUpdateStatus:         true,
-			jobPresentInCJActiveStatus: false,
-		},
 		"prev ran but done, is time, F": {
 			concurrencyPolicy:          "Forbid",
 			schedule:                   onTheHour,
@@ -1188,8 +1163,6 @@ func TestControllerV2SyncCronJob(t *testing.T) {
 		tc := tc
 
 		t.Run(name, func(t *testing.T) {
-			defer featuregatetesting.SetFeatureGateDuringTest(t, feature.DefaultFeatureGate, features.CronJobTimeZone, tc.enableTimeZone)()
-
 			cj := cronJob()
 			cj.Spec.ConcurrencyPolicy = tc.concurrencyPolicy
 			cj.Spec.Suspend = &tc.suspend
@@ -1264,8 +1237,7 @@ func TestControllerV2SyncCronJob(t *testing.T) {
 					return tc.now
 				},
 			}
-			cjCopy := cj.DeepCopy()
-			requeueAfter, updateStatus, err := jm.syncCronJob(context.TODO(), cjCopy, js)
+			cjCopy, requeueAfter, updateStatus, err := jm.syncCronJob(context.TODO(), &cj, js)
 			if tc.expectErr && err == nil {
 				t.Errorf("%s: expected error got none with requeueAfter time: %#v", name, requeueAfter)
 			}
@@ -1573,9 +1545,12 @@ func TestControllerV2UpdateCronJob(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
+			logger, ctx := ktesting.NewTestContext(t)
+			ctx, cancel := context.WithCancel(ctx)
+			defer cancel()
 			kubeClient := fake.NewSimpleClientset()
 			sharedInformers := informers.NewSharedInformerFactory(kubeClient, controller.NoResyncPeriodFunc())
-			jm, err := NewControllerV2(sharedInformers.Batch().V1().Jobs(), sharedInformers.Batch().V1().CronJobs(), kubeClient)
+			jm, err := NewControllerV2(ctx, sharedInformers.Batch().V1().Jobs(), sharedInformers.Batch().V1().CronJobs(), kubeClient)
 			if err != nil {
 				t.Errorf("unexpected error %v", err)
 				return
@@ -1587,7 +1562,7 @@ func TestControllerV2UpdateCronJob(t *testing.T) {
 			jm.cronJobControl = &fakeCJControl{}
 			jm.recorder = record.NewFakeRecorder(10)
 
-			jm.updateCronJob(tt.oldCronJob, tt.newCronJob)
+			jm.updateCronJob(logger, tt.oldCronJob, tt.newCronJob)
 			if queue.delay.Seconds() != tt.expectedDelay.Seconds() {
 				t.Errorf("Expected delay %#v got %#v", tt.expectedDelay.Seconds(), queue.delay.Seconds())
 			}
@@ -1670,12 +1645,15 @@ func TestControllerV2GetJobsToBeReconciled(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
+			_, ctx := ktesting.NewTestContext(t)
+			ctx, cancel := context.WithCancel(ctx)
+			defer cancel()
 			kubeClient := fake.NewSimpleClientset()
 			sharedInformers := informers.NewSharedInformerFactory(kubeClient, controller.NoResyncPeriodFunc())
 			for _, job := range tt.jobs {
 				sharedInformers.Batch().V1().Jobs().Informer().GetIndexer().Add(job)
 			}
-			jm, err := NewControllerV2(sharedInformers.Batch().V1().Jobs(), sharedInformers.Batch().V1().CronJobs(), kubeClient)
+			jm, err := NewControllerV2(ctx, sharedInformers.Batch().V1().Jobs(), sharedInformers.Batch().V1().CronJobs(), kubeClient)
 			if err != nil {
 				t.Errorf("unexpected error %v", err)
 				return
@@ -1692,220 +1670,3 @@ func TestControllerV2GetJobsToBeReconciled(t *testing.T) {
 		})
 	}
 }
-
-func TestControllerV2CleanupFinishedJobs(t *testing.T) {
-	tests := []struct {
-		name                string
-		now                 time.Time
-		cronJob             *batchv1.CronJob
-		finishedJobs        []*batchv1.Job
-		jobCreateError      error
-		expectedDeletedJobs []string
-	}{
-		{
-			name: "jobs are still deleted when a cronjob can't create jobs due to jobs quota being reached (avoiding a deadlock)",
-			now:  *justAfterTheHour(),
-			cronJob: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{Namespace: "foo-ns", Name: "fooer"},
-				Spec: batchv1.CronJobSpec{
-					Schedule:                   onTheHour,
-					SuccessfulJobsHistoryLimit: pointer.Int32(1),
-					JobTemplate: batchv1.JobTemplateSpec{
-						ObjectMeta: metav1.ObjectMeta{Labels: map[string]string{"key": "value"}},
-					},
-				},
-				Status: batchv1.CronJobStatus{LastScheduleTime: &metav1.Time{Time: justAfterThePriorHour()}},
-			},
-			finishedJobs: []*batchv1.Job{
-				{
-					ObjectMeta: metav1.ObjectMeta{
-						Namespace:       "foo-ns",
-						Name:            "finished-job-started-hour-ago",
-						OwnerReferences: []metav1.OwnerReference{{Name: "fooer", Controller: pointer.Bool(true)}},
-					},
-					Status: batchv1.JobStatus{StartTime: &metav1.Time{Time: justBeforeThePriorHour()}},
-				},
-				{
-					ObjectMeta: metav1.ObjectMeta{
-						Namespace:       "foo-ns",
-						Name:            "finished-job-started-minute-ago",
-						OwnerReferences: []metav1.OwnerReference{{Name: "fooer", Controller: pointer.Bool(true)}},
-					},
-					Status: batchv1.JobStatus{StartTime: &metav1.Time{Time: justBeforeTheHour()}},
-				},
-			},
-			jobCreateError:      errors.NewInternalError(fmt.Errorf("quota for # of jobs reached")),
-			expectedDeletedJobs: []string{"finished-job-started-hour-ago"},
-		},
-		{
-			name: "jobs are not deleted if history limit not reached",
-			now:  justBeforeTheHour(),
-			cronJob: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{Namespace: "foo-ns", Name: "fooer"},
-				Spec: batchv1.CronJobSpec{
-					Schedule:                   onTheHour,
-					SuccessfulJobsHistoryLimit: pointer.Int32(2),
-					JobTemplate: batchv1.JobTemplateSpec{
-						ObjectMeta: metav1.ObjectMeta{Labels: map[string]string{"key": "value"}},
-					},
-				},
-				Status: batchv1.CronJobStatus{LastScheduleTime: &metav1.Time{Time: justAfterThePriorHour()}},
-			},
-			finishedJobs: []*batchv1.Job{
-				{
-					ObjectMeta: metav1.ObjectMeta{
-						Namespace:       "foo-ns",
-						Name:            "finished-job-started-hour-ago",
-						OwnerReferences: []metav1.OwnerReference{{Name: "fooer", Controller: pointer.Bool(true)}},
-					},
-					Status: batchv1.JobStatus{StartTime: &metav1.Time{Time: justBeforeThePriorHour()}},
-				},
-			},
-			jobCreateError:      nil,
-			expectedDeletedJobs: []string{},
-		},
-	}
-
-	for _, tt := range tests {
-		t.Run(tt.name, func(t *testing.T) {
-			_, ctx := ktesting.NewTestContext(t)
-
-			for _, job := range tt.finishedJobs {
-				job.Status.Conditions = []batchv1.JobCondition{{Type: batchv1.JobComplete, Status: v1.ConditionTrue}}
-			}
-
-			client := fake.NewSimpleClientset()
-
-			informerFactory := informers.NewSharedInformerFactory(client, controller.NoResyncPeriodFunc())
-			_ = informerFactory.Batch().V1().CronJobs().Informer().GetIndexer().Add(tt.cronJob)
-			for _, job := range tt.finishedJobs {
-				_ = informerFactory.Batch().V1().Jobs().Informer().GetIndexer().Add(job)
-			}
-
-			jm, err := NewControllerV2(informerFactory.Batch().V1().Jobs(), informerFactory.Batch().V1().CronJobs(), client)
-			if err != nil {
-				t.Errorf("unexpected error %v", err)
-				return
-			}
-			jobControl := &fakeJobControl{CreateErr: tt.jobCreateError}
-			jm.jobControl = jobControl
-			jm.now = func() time.Time {
-				return tt.now
-			}
-
-			jm.enqueueController(tt.cronJob)
-			jm.processNextWorkItem(ctx)
-
-			if len(tt.expectedDeletedJobs) != len(jobControl.DeleteJobName) {
-				t.Fatalf("expected '%v' jobs to be deleted, instead deleted '%s'", tt.expectedDeletedJobs, jobControl.DeleteJobName)
-			}
-			sort.Strings(jobControl.DeleteJobName)
-			sort.Strings(tt.expectedDeletedJobs)
-			for i, deletedJob := range jobControl.DeleteJobName {
-				if deletedJob != tt.expectedDeletedJobs[i] {
-					t.Fatalf("expected '%v' jobs to be deleted, instead deleted '%s'", tt.expectedDeletedJobs, jobControl.DeleteJobName)
-				}
-			}
-		})
-	}
-}
-
-// TestControllerV2JobAlreadyExistsButNotInActiveStatus validates that an already created job that was not added to the status
-// of a CronJob initially will be added back on the next sync. Previously, if we failed to update the status after creating a job,
-// cronjob controller would retry continuously because it would attempt to create a job that already exists.
-func TestControllerV2JobAlreadyExistsButNotInActiveStatus(t *testing.T) {
-	_, ctx := ktesting.NewTestContext(t)
-
-	cj := cronJob()
-	cj.Spec.ConcurrencyPolicy = "Forbid"
-	cj.Spec.Schedule = everyHour
-	cj.Status.LastScheduleTime = &metav1.Time{Time: justBeforeThePriorHour()}
-	cj.Status.Active = []v1.ObjectReference{}
-	cjCopy := cj.DeepCopy()
-
-	job, err := getJobFromTemplate2(&cj, justAfterThePriorHour())
-	if err != nil {
-		t.Fatalf("Unexpected error creating a job from template: %v", err)
-	}
-	job.UID = "1234"
-	job.Namespace = cj.Namespace
-
-	client := fake.NewSimpleClientset(cjCopy, job)
-	informerFactory := informers.NewSharedInformerFactory(client, controller.NoResyncPeriodFunc())
-	_ = informerFactory.Batch().V1().CronJobs().Informer().GetIndexer().Add(cjCopy)
-
-	jm, err := NewControllerV2(informerFactory.Batch().V1().Jobs(), informerFactory.Batch().V1().CronJobs(), client)
-	if err != nil {
-		t.Fatalf("unexpected error %v", err)
-	}
-
-	jobControl := &fakeJobControl{Job: job, CreateErr: errors.NewAlreadyExists(schema.GroupResource{Resource: "job", Group: "batch"}, "")}
-	jm.jobControl = jobControl
-	cronJobControl := &fakeCJControl{}
-	jm.cronJobControl = cronJobControl
-	jm.now = justBeforeTheHour
-
-	jm.enqueueController(cjCopy)
-	jm.processNextWorkItem(ctx)
-
-	if len(cronJobControl.Updates) != 1 {
-		t.Fatalf("Unexpected updates to cronjob, got: %d, expected 1", len(cronJobControl.Updates))
-	}
-	if len(cronJobControl.Updates[0].Status.Active) != 1 {
-		t.Errorf("Unexpected active jobs count, got: %d, expected 1", len(cronJobControl.Updates[0].Status.Active))
-	}
-
-	expectedActiveRef, err := getRef(job)
-	if err != nil {
-		t.Fatalf("Error getting expected job ref: %v", err)
-	}
-	if !reflect.DeepEqual(cronJobControl.Updates[0].Status.Active[0], *expectedActiveRef) {
-		t.Errorf("Unexpected job reference in cronjob active list, got: %v, expected: %v", cronJobControl.Updates[0].Status.Active[0], expectedActiveRef)
-	}
-}
-
-// TestControllerV2JobAlreadyExistsButDifferentOwnner validates that an already created job
-// not owned by the cronjob controller is ignored.
-func TestControllerV2JobAlreadyExistsButDifferentOwner(t *testing.T) {
-	_, ctx := ktesting.NewTestContext(t)
-
-	cj := cronJob()
-	cj.Spec.ConcurrencyPolicy = "Forbid"
-	cj.Spec.Schedule = everyHour
-	cj.Status.LastScheduleTime = &metav1.Time{Time: justBeforeThePriorHour()}
-	cj.Status.Active = []v1.ObjectReference{}
-	cjCopy := cj.DeepCopy()
-
-	job, err := getJobFromTemplate2(&cj, justAfterThePriorHour())
-	if err != nil {
-		t.Fatalf("Unexpected error creating a job from template: %v", err)
-	}
-	job.UID = "1234"
-	job.Namespace = cj.Namespace
-
-	// remove owners for this test since we are testing that jobs not belonging to cronjob
-	// controller are safely ignored
-	job.OwnerReferences = []metav1.OwnerReference{}
-
-	client := fake.NewSimpleClientset(cjCopy, job)
-	informerFactory := informers.NewSharedInformerFactory(client, controller.NoResyncPeriodFunc())
-	_ = informerFactory.Batch().V1().CronJobs().Informer().GetIndexer().Add(cjCopy)
-
-	jm, err := NewControllerV2(informerFactory.Batch().V1().Jobs(), informerFactory.Batch().V1().CronJobs(), client)
-	if err != nil {
-		t.Fatalf("unexpected error %v", err)
-	}
-
-	jobControl := &fakeJobControl{Job: job, CreateErr: errors.NewAlreadyExists(schema.GroupResource{Resource: "job", Group: "batch"}, "")}
-	jm.jobControl = jobControl
-	cronJobControl := &fakeCJControl{}
-	jm.cronJobControl = cronJobControl
-	jm.now = justBeforeTheHour
-
-	jm.enqueueController(cjCopy)
-	jm.processNextWorkItem(ctx)
-
-	if len(cronJobControl.Updates) != 0 {
-		t.Fatalf("Unexpected updates to cronjob, got: %d, expected 0", len(cronJobControl.Updates))
-	}
-}
