@@ -27,7 +27,7 @@ import (
 	discovery "k8s.io/api/discovery/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	utilpointer "k8s.io/utils/pointer"
+	"k8s.io/utils/pointer"
 )
 
 func TestAddHints(t *testing.T) {
@@ -65,8 +65,8 @@ func TestAddHints(t *testing.T) {
 			ToCreate: []*discovery.EndpointSlice{{
 				Endpoints: []discovery.Endpoint{{
 					Addresses:  []string{"10.1.2.3"},
-					Zone:       utilpointer.StringPtr("zone-a"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-a"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}},
 			}},
 		},
@@ -74,8 +74,8 @@ func TestAddHints(t *testing.T) {
 		expectedSlicesToCreate: []*discovery.EndpointSlice{{
 			Endpoints: []discovery.Endpoint{{
 				Addresses:  []string{"10.1.2.3"},
-				Zone:       utilpointer.StringPtr("zone-a"),
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Zone:       pointer.String("zone-a"),
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}},
 		}},
 		expectedSlicesToUpdate: []*discovery.EndpointSlice{},
@@ -99,12 +99,12 @@ func TestAddHints(t *testing.T) {
 			ToCreate: []*discovery.EndpointSlice{{
 				Endpoints: []discovery.Endpoint{{
 					Addresses:  []string{"10.1.2.3"},
-					Zone:       utilpointer.StringPtr("zone-a"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-a"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.1.2.4"},
-					Zone:       utilpointer.StringPtr("zone-b"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-b"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}},
 			}},
 		},
@@ -112,12 +112,12 @@ func TestAddHints(t *testing.T) {
 		expectedSlicesToCreate: []*discovery.EndpointSlice{{
 			Endpoints: []discovery.Endpoint{{
 				Addresses:  []string{"10.1.2.3"},
-				Zone:       utilpointer.StringPtr("zone-a"),
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Zone:       pointer.String("zone-a"),
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.1.2.4"},
-				Zone:       utilpointer.StringPtr("zone-b"),
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Zone:       pointer.String("zone-b"),
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}},
 		}},
 		expectedSlicesToUpdate: []*discovery.EndpointSlice{},
@@ -140,12 +140,12 @@ func TestAddHints(t *testing.T) {
 			ToCreate: []*discovery.EndpointSlice{{
 				Endpoints: []discovery.Endpoint{{
 					Addresses:  []string{"10.1.2.3"},
-					Zone:       utilpointer.StringPtr("zone-a"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-a"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.1.2.4"},
-					Zone:       utilpointer.StringPtr("zone-b"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-b"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}},
 			}},
 		},
@@ -158,14 +158,14 @@ func TestAddHints(t *testing.T) {
 		expectedSlicesToCreate: []*discovery.EndpointSlice{{
 			Endpoints: []discovery.Endpoint{{
 				Addresses:  []string{"10.1.2.3"},
-				Zone:       utilpointer.StringPtr("zone-a"),
+				Zone:       pointer.String("zone-a"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-a"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.1.2.4"},
-				Zone:       utilpointer.StringPtr("zone-b"),
+				Zone:       pointer.String("zone-b"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-b"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}},
 		}},
 		expectedSlicesToUpdate: []*discovery.EndpointSlice{},
@@ -188,19 +188,19 @@ func TestAddHints(t *testing.T) {
 			ToCreate: []*discovery.EndpointSlice{{
 				Endpoints: []discovery.Endpoint{{
 					Addresses:  []string{"10.1.2.3"},
-					Zone:       utilpointer.StringPtr("zone-a"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-a"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.1.2.4"},
-					Zone:       utilpointer.StringPtr("zone-b"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-b"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.1.2.5"},
-					Zone:       utilpointer.StringPtr("zone-b"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(false)},
+					Zone:       pointer.String("zone-b"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(false)},
 				}, {
 					Addresses: []string{"10.1.2.6"},
-					Zone:      utilpointer.StringPtr("zone-b"),
+					Zone:      pointer.String("zone-b"),
 				}},
 			}},
 		},
@@ -213,22 +213,22 @@ func TestAddHints(t *testing.T) {
 		expectedSlicesToCreate: []*discovery.EndpointSlice{{
 			Endpoints: []discovery.Endpoint{{
 				Addresses:  []string{"10.1.2.3"},
-				Zone:       utilpointer.StringPtr("zone-a"),
+				Zone:       pointer.String("zone-a"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-a"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.1.2.4"},
-				Zone:       utilpointer.StringPtr("zone-b"),
+				Zone:       pointer.String("zone-b"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-b"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.1.2.5"},
-				Zone:       utilpointer.StringPtr("zone-b"),
+				Zone:       pointer.String("zone-b"),
 				Hints:      nil,
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(false)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(false)},
 			}, {
 				Addresses: []string{"10.1.2.6"},
-				Zone:      utilpointer.StringPtr("zone-b"),
+				Zone:      pointer.String("zone-b"),
 				Hints:     nil,
 			}},
 		}},
@@ -253,51 +253,51 @@ func TestAddHints(t *testing.T) {
 			ToCreate: []*discovery.EndpointSlice{{
 				Endpoints: []discovery.Endpoint{{
 					Addresses:  []string{"10.1.2.3"},
-					Zone:       utilpointer.StringPtr("zone-a"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-a"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.1.2.4"},
-					Zone:       utilpointer.StringPtr("zone-b"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-b"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}},
 			}, {
 				Endpoints: []discovery.Endpoint{{
 					Addresses:  []string{"10.1.3.3"},
-					Zone:       utilpointer.StringPtr("zone-c"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-c"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.1.3.4"},
-					Zone:       utilpointer.StringPtr("zone-c"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-c"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.1.3.4"},
-					Zone:       utilpointer.StringPtr("zone-a"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-a"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}},
 			}},
 			ToUpdate: []*discovery.EndpointSlice{{
 				Endpoints: []discovery.Endpoint{{
 					Addresses:  []string{"10.2.2.3"},
-					Zone:       utilpointer.StringPtr("zone-a"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-a"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.2.2.4"},
-					Zone:       utilpointer.StringPtr("zone-a"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-a"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}},
 			}, {
 				Endpoints: []discovery.Endpoint{{
 					Addresses:  []string{"10.2.3.3"},
-					Zone:       utilpointer.StringPtr("zone-b"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-b"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.2.3.4"},
-					Zone:       utilpointer.StringPtr("zone-c"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-c"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}, {
 					Addresses:  []string{"10.2.3.4"},
-					Zone:       utilpointer.StringPtr("zone-a"),
-					Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+					Zone:       pointer.String("zone-a"),
+					Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 				}},
 			}},
 		},
@@ -311,61 +311,61 @@ func TestAddHints(t *testing.T) {
 		expectedSlicesToCreate: []*discovery.EndpointSlice{{
 			Endpoints: []discovery.Endpoint{{
 				Addresses:  []string{"10.1.2.3"},
-				Zone:       utilpointer.StringPtr("zone-a"),
+				Zone:       pointer.String("zone-a"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-b"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.1.2.4"},
-				Zone:       utilpointer.StringPtr("zone-b"),
+				Zone:       pointer.String("zone-b"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-b"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}},
 		}, {
 			Endpoints: []discovery.Endpoint{{
 				Addresses:  []string{"10.1.3.3"},
-				Zone:       utilpointer.StringPtr("zone-c"),
+				Zone:       pointer.String("zone-c"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-c"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.1.3.4"},
-				Zone:       utilpointer.StringPtr("zone-c"),
+				Zone:       pointer.String("zone-c"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-c"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.1.3.4"},
-				Zone:       utilpointer.StringPtr("zone-a"),
+				Zone:       pointer.String("zone-a"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-a"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}},
 		}},
 		expectedSlicesToUpdate: []*discovery.EndpointSlice{{
 			Endpoints: []discovery.Endpoint{{
 				Addresses:  []string{"10.2.2.3"},
-				Zone:       utilpointer.StringPtr("zone-a"),
+				Zone:       pointer.String("zone-a"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-a"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.2.2.4"},
-				Zone:       utilpointer.StringPtr("zone-a"),
+				Zone:       pointer.String("zone-a"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-a"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}},
 		}, {
 			Endpoints: []discovery.Endpoint{{
 				Addresses:  []string{"10.2.3.3"},
-				Zone:       utilpointer.StringPtr("zone-b"),
+				Zone:       pointer.String("zone-b"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-b"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.2.3.4"},
-				Zone:       utilpointer.StringPtr("zone-c"),
+				Zone:       pointer.String("zone-c"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-c"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}, {
 				Addresses:  []string{"10.2.3.4"},
-				Zone:       utilpointer.StringPtr("zone-a"),
+				Zone:       pointer.String("zone-a"),
 				Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-a"}}},
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.BoolPtr(true)},
+				Conditions: discovery.EndpointConditions{Ready: pointer.Bool(true)},
 			}},
 		}},
 		expectedEvents: []*EventBuilder{
@@ -625,72 +625,6 @@ func TestSetNodes(t *testing.T) {
 	}
 }
 
-func TestTopologyCacheRace(t *testing.T) {
-	sliceInfo := &SliceInfo{
-		ServiceKey:  "ns/svc",
-		AddressType: discovery.AddressTypeIPv4,
-		ToCreate: []*discovery.EndpointSlice{{
-			Endpoints: []discovery.Endpoint{{
-				Addresses:  []string{"10.1.2.3"},
-				Zone:       utilpointer.String("zone-a"),
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.Bool(true)},
-			}, {
-				Addresses:  []string{"10.1.2.4"},
-				Zone:       utilpointer.String("zone-b"),
-				Conditions: discovery.EndpointConditions{Ready: utilpointer.Bool(true)},
-			}},
-		}}}
-	type nodeInfo struct {
-		zone  string
-		cpu   resource.Quantity
-		ready v1.ConditionStatus
-	}
-	nodeInfos := []nodeInfo{
-		{zone: "zone-a", cpu: resource.MustParse("1000m"), ready: v1.ConditionTrue},
-		{zone: "zone-a", cpu: resource.MustParse("1000m"), ready: v1.ConditionTrue},
-		{zone: "zone-a", cpu: resource.MustParse("1000m"), ready: v1.ConditionTrue},
-		{zone: "zone-a", cpu: resource.MustParse("2000m"), ready: v1.ConditionTrue},
-		{zone: "zone-b", cpu: resource.MustParse("3000m"), ready: v1.ConditionTrue},
-		{zone: "zone-b", cpu: resource.MustParse("1500m"), ready: v1.ConditionTrue},
-		{zone: "zone-c", cpu: resource.MustParse("500m"), ready: v1.ConditionTrue},
-	}
-
-	cache := NewTopologyCache()
-	nodes := []*v1.Node{}
-	for _, node := range nodeInfos {
-		labels := map[string]string{}
-		if node.zone != "" {
-			labels[v1.LabelTopologyZone] = node.zone
-		}
-		conditions := []v1.NodeCondition{{
-			Type:   v1.NodeReady,
-			Status: node.ready,
-		}}
-		allocatable := v1.ResourceList{
-			v1.ResourceCPU: node.cpu,
-		}
-		nodes = append(nodes, &v1.Node{
-			ObjectMeta: metav1.ObjectMeta{
-				Labels: labels,
-			},
-			Status: v1.NodeStatus{
-				Allocatable: allocatable,
-				Conditions:  conditions,
-			},
-		})
-	}
-
-	go func() {
-		cache.SetNodes(nodes)
-	}()
-	go func() {
-		cache.AddHints(sliceInfo)
-	}()
-	go func() {
-		cache.HasPopulatedHints(sliceInfo.ServiceKey)
-	}()
-}
-
 // Test Helpers
 
 func expectEquivalentSlices(t *testing.T, actualSlices, expectedSlices []*discovery.EndpointSlice) {
