@@ -340,8 +340,8 @@ func getPersistentVolumeClaims(set *apps.StatefulSet, pod *v1.Pod) map[string]v1
 	templates := set.Spec.VolumeClaimTemplates
 	claims := make(map[string]v1.PersistentVolumeClaim, len(templates))
 	for i := range templates {
-		claim := templates[i].DeepCopy()
-		claim.Name = getPersistentVolumeClaimName(set, claim, ordinal)
+		claim := templates[i]
+		claim.Name = getPersistentVolumeClaimName(set, &claim, ordinal)
 		claim.Namespace = set.Namespace
 		if claim.Labels != nil {
 			for key, value := range set.Spec.Selector.MatchLabels {
@@ -350,7 +350,7 @@ func getPersistentVolumeClaims(set *apps.StatefulSet, pod *v1.Pod) map[string]v1
 		} else {
 			claim.Labels = set.Spec.Selector.MatchLabels
 		}
-		claims[templates[i].Name] = *claim
+		claims[templates[i].Name] = claim
 	}
 	return claims
 }
@@ -413,6 +413,11 @@ func isCreated(pod *v1.Pod) bool {
 	return pod.Status.Phase != ""
 }
 
+// isPending returns true if pod has a Phase of PodPending
+func isPending(pod *v1.Pod) bool {
+	return pod.Status.Phase == v1.PodPending
+}
+
 // isFailed returns true if pod has a Phase of PodFailed
 func isFailed(pod *v1.Pod) bool {
 	return pod.Status.Phase == v1.PodFailed
@@ -610,23 +615,6 @@ func (ao ascendingOrdinal) Less(i, j int) bool {
 	return getOrdinal(ao[i]) < getOrdinal(ao[j])
 }
 
-// descendingOrdinal is a sort.Interface that Sorts a list of Pods based on the ordinals extracted
-// from the Pod. Pod's that have not been constructed by StatefulSet's have an ordinal of -1, and are therefore pushed
-// to the end of the list.
-type descendingOrdinal []*v1.Pod
-
-func (do descendingOrdinal) Len() int {
-	return len(do)
-}
-
-func (do descendingOrdinal) Swap(i, j int) {
-	do[i], do[j] = do[j], do[i]
-}
-
-func (do descendingOrdinal) Less(i, j int) bool {
-	return getOrdinal(do[i]) > getOrdinal(do[j])
-}
-
 // getStatefulSetMaxUnavailable calculates the real maxUnavailable number according to the replica count
 // and maxUnavailable from rollingUpdateStrategy. The number defaults to 1 if the maxUnavailable field is
 // not set, and it will be round down to at least 1 if the maxUnavailable value is a percentage.
