@@ -20,6 +20,7 @@ limitations under the License.
 package reconciler
 
 import (
+	"context"
 	"fmt"
 	"strings"
 	"time"
@@ -53,7 +54,7 @@ type Reconciler interface {
 	// if volumes that should be attached are attached and volumes that should
 	// be detached are detached. If not, it will trigger attach/detach
 	// operations to rectify.
-	Run(stopCh <-chan struct{})
+	Run(ctx context.Context)
 }
 
 // NewReconciler returns a new instance of Reconciler that waits loopPeriod
@@ -105,24 +106,24 @@ type reconciler struct {
 	recorder                  record.EventRecorder
 }
 
-func (rc *reconciler) Run(stopCh <-chan struct{}) {
-	wait.Until(rc.reconciliationLoopFunc(), rc.loopPeriod, stopCh)
+func (rc *reconciler) Run(ctx context.Context) {
+	wait.UntilWithContext(ctx, rc.reconciliationLoopFunc(ctx), rc.loopPeriod)
 }
 
 // reconciliationLoopFunc this can be disabled via cli option disableReconciliation.
 // It periodically checks whether the attached volumes from actual state
 // are still attached to the node and update the status if they are not.
-func (rc *reconciler) reconciliationLoopFunc() func() {
-	return func() {
-
-		rc.reconcile()
+func (rc *reconciler) reconciliationLoopFunc(ctx context.Context) func(context.Context) {
+	return func(ctx context.Context) {
 
+		rc.reconcile(ctx)
+		logger := klog.FromContext(ctx)
 		if rc.disableReconciliationSync {
-			klog.V(5).Info("Skipping reconciling attached volumes still attached since it is disabled via the command line.")
+			logger.V(5).Info("Skipping reconciling attached volumes still attached since it is disabled via the command line")
 		} else if rc.syncDuration < time.Second {
-			klog.V(5).Info("Skipping reconciling attached volumes still attached since it is set to less than one second via the command line.")
+			logger.V(5).Info("Skipping reconciling attached volumes still attached since it is set to less than one second via the command line")
 		} else if time.Since(rc.timeOfLastSync) > rc.syncDuration {
-			klog.V(5).Info("Starting reconciling attached volumes still attached")
+			logger.V(5).Info("Starting reconciling attached volumes still attached")
 			rc.sync()
 		}
 	}
@@ -164,11 +165,12 @@ func (rc *reconciler) nodeIsHealthy(nodeName types.NodeName) (bool, error) {
 	return nodeutil.IsNodeReady(node), nil
 }
 
-func (rc *reconciler) reconcile() {
+func (rc *reconciler) reconcile(ctx context.Context) {
 	// Detaches are triggered before attaches so that volumes referenced by
 	// pods that are rescheduled to a different node are detached first.
 
 	// Ensure volumes that should be detached are detached.
+	logger := klog.FromContext(ctx)
 	for _, attachedVolume := range rc.actualStateOfWorld.GetAttachedVolumes() {
 		if !rc.desiredStateOfWorld.VolumeExists(
 			attachedVolume.VolumeName, attachedVolume.NodeName) {
@@ -182,12 +184,12 @@ func (rc *reconciler) reconcile() {
 			// allows multi attach across different nodes.
 			if util.IsMultiAttachAllowed(attachedVolume.VolumeSpec) {
 				if !rc.attacherDetacher.IsOperationSafeToRetry(attachedVolume.VolumeName, "" /* podName */, attachedVolume.NodeName, operationexecutor.DetachOperationName) {
-					klog.V(10).Infof("Operation for volume %q is already running or still in exponential backoff for node %q. Can't start detach", attachedVolume.VolumeName, attachedVolume.NodeName)
+					logger.V(10).Info("Operation for volume is already running or still in exponential backoff for node. Can't start detach", "node", klog.KRef("", string(attachedVolume.NodeName)), "volumeName", attachedVolume.VolumeName)
 					continue
 				}
 			} else {
 				if !rc.attacherDetacher.IsOperationSafeToRetry(attachedVolume.VolumeName, "" /* podName */, "" /* nodeName */, operationexecutor.DetachOperationName) {
-					klog.V(10).Infof("Operation for volume %q is already running or still in exponential backoff in the cluster. Can't start detach for %q", attachedVolume.VolumeName, attachedVolume.NodeName)
+					logger.V(10).Info("Operation for volume is already running or still in exponential backoff in the cluster. Can't start detach for node", "node", klog.KRef("", string(attachedVolume.NodeName)), "volumeName", attachedVolume.VolumeName)
 					continue
 				}
 			}
@@ -201,14 +203,14 @@ func (rc *reconciler) reconcile() {
 			// See https://github.com/kubernetes/kubernetes/issues/93902
 			attachState := rc.actualStateOfWorld.GetAttachState(attachedVolume.VolumeName, attachedVolume.NodeName)
 			if attachState == cache.AttachStateDetached {
-				klog.V(5).InfoS("Volume detached--skipping", "volume", attachedVolume)
+				logger.V(5).Info("Volume detached--skipping", "volume", attachedVolume)
 				continue
 			}
 
 			// Set the detach request time
-			elapsedTime, err := rc.actualStateOfWorld.SetDetachRequestTime(attachedVolume.VolumeName, attachedVolume.NodeName)
+			elapsedTime, err := rc.actualStateOfWorld.SetDetachRequestTime(logger, attachedVolume.VolumeName, attachedVolume.NodeName)
 			if err != nil {
-				klog.Errorf("Cannot trigger detach because it fails to set detach request time with error %v", err)
+				logger.Error(err, "Cannot trigger detach because it fails to set detach request time with error")
 				continue
 			}
 			// Check whether timeout has reached the maximum waiting time
@@ -216,7 +218,7 @@ func (rc *reconciler) reconcile() {
 
 			isHealthy, err := rc.nodeIsHealthy(attachedVolume.NodeName)
 			if err != nil {
-				klog.Errorf("failed to get health of node %s: %s", attachedVolume.NodeName, err.Error())
+				logger.Error(err, "Failed to get health of node", "node", klog.KRef("", string(attachedVolume.NodeName)))
 			}
 
 			// Force detach volumes from unhealthy nodes after maxWaitForUnmountDuration.
@@ -224,13 +226,13 @@ func (rc *reconciler) reconcile() {
 
 			hasOutOfServiceTaint, err := rc.hasOutOfServiceTaint(attachedVolume.NodeName)
 			if err != nil {
-				klog.Errorf("failed to get taint specs for node %s: %s", attachedVolume.NodeName, err.Error())
+				logger.Error(err, "Failed to get taint specs for node", "node", klog.KRef("", string(attachedVolume.NodeName)))
 			}
 
 			// Check whether volume is still mounted. Skip detach if it is still mounted unless force detach timeout
 			// or the node has `node.kubernetes.io/out-of-service` taint.
 			if attachedVolume.MountedByNode && !forceDetach && !hasOutOfServiceTaint {
-				klog.V(5).InfoS("Cannot detach volume because it is still mounted", "volume", attachedVolume)
+				logger.V(5).Info("Cannot detach volume because it is still mounted", "volume", attachedVolume)
 				continue
 			}
 
@@ -240,77 +242,77 @@ func (rc *reconciler) reconcile() {
 			// has the correct volume attachment information.
 			err = rc.actualStateOfWorld.RemoveVolumeFromReportAsAttached(attachedVolume.VolumeName, attachedVolume.NodeName)
 			if err != nil {
-				klog.V(5).Infof("RemoveVolumeFromReportAsAttached failed while removing volume %q from node %q with: %v",
-					attachedVolume.VolumeName,
-					attachedVolume.NodeName,
-					err)
+				logger.V(5).Info("RemoveVolumeFromReportAsAttached failed while removing volume from node",
+					"node", klog.KRef("", string(attachedVolume.NodeName)),
+					"volumeName", attachedVolume.VolumeName,
+					"err", err)
 			}
 
 			// Update Node Status to indicate volume is no longer safe to mount.
-			err = rc.nodeStatusUpdater.UpdateNodeStatusForNode(attachedVolume.NodeName)
+			err = rc.nodeStatusUpdater.UpdateNodeStatusForNode(logger, attachedVolume.NodeName)
 			if err != nil {
 				// Skip detaching this volume if unable to update node status
-				klog.ErrorS(err, "UpdateNodeStatusForNode failed while attempting to report volume as attached", "volume", attachedVolume)
+				logger.Error(err, "UpdateNodeStatusForNode failed while attempting to report volume as attached", "volume", attachedVolume)
 				// Add volume back to ReportAsAttached if UpdateNodeStatusForNode call failed so that node status updater will add it back to VolumeAttached list.
 				// It is needed here too because DetachVolume is not call actually and we keep the data consistency for every reconcile.
-				rc.actualStateOfWorld.AddVolumeToReportAsAttached(attachedVolume.VolumeName, attachedVolume.NodeName)
+				rc.actualStateOfWorld.AddVolumeToReportAsAttached(logger, attachedVolume.VolumeName, attachedVolume.NodeName)
 				continue
 			}
 
 			// Trigger detach volume which requires verifying safe to detach step
 			// If timeout is true, skip verifySafeToDetach check
 			// If the node has node.kubernetes.io/out-of-service taint with NoExecute effect, skip verifySafeToDetach check
-			klog.V(5).InfoS("Starting attacherDetacher.DetachVolume", "volume", attachedVolume)
+			logger.V(5).Info("Starting attacherDetacher.DetachVolume", "volume", attachedVolume)
 			if hasOutOfServiceTaint {
-				klog.V(4).Infof("node %q has out-of-service taint", attachedVolume.NodeName)
+				logger.V(4).Info("node has out-of-service taint", "node", klog.KRef("", string(attachedVolume.NodeName)))
 			}
 			verifySafeToDetach := !(timeout || hasOutOfServiceTaint)
-			err = rc.attacherDetacher.DetachVolume(attachedVolume.AttachedVolume, verifySafeToDetach, rc.actualStateOfWorld)
+			err = rc.attacherDetacher.DetachVolume(logger, attachedVolume.AttachedVolume, verifySafeToDetach, rc.actualStateOfWorld)
 			if err == nil {
 				if !timeout {
-					klog.InfoS("attacherDetacher.DetachVolume started", "volume", attachedVolume)
+					logger.Info("attacherDetacher.DetachVolume started", "volume", attachedVolume)
 				} else {
 					metrics.RecordForcedDetachMetric()
-					klog.InfoS("attacherDetacher.DetachVolume started: this volume is not safe to detach, but maxWaitForUnmountDuration expired, force detaching", "duration", rc.maxWaitForUnmountDuration, "volume", attachedVolume)
+					logger.Info("attacherDetacher.DetachVolume started: this volume is not safe to detach, but maxWaitForUnmountDuration expired, force detaching", "duration", rc.maxWaitForUnmountDuration, "volume", attachedVolume)
 				}
 			}
 			if err != nil {
 				// Add volume back to ReportAsAttached if DetachVolume call failed so that node status updater will add it back to VolumeAttached list.
 				// This function is also called during executing the volume detach operation in operation_generoator.
 				// It is needed here too because DetachVolume call might fail before executing the actual operation in operation_executor (e.g., cannot find volume plugin etc.)
-				rc.actualStateOfWorld.AddVolumeToReportAsAttached(attachedVolume.VolumeName, attachedVolume.NodeName)
+				rc.actualStateOfWorld.AddVolumeToReportAsAttached(logger, attachedVolume.VolumeName, attachedVolume.NodeName)
 
 				if !exponentialbackoff.IsExponentialBackoff(err) {
 					// Ignore exponentialbackoff.IsExponentialBackoff errors, they are expected.
 					// Log all other errors.
-					klog.ErrorS(err, "attacherDetacher.DetachVolume failed to start", "volume", attachedVolume)
+					logger.Error(err, "attacherDetacher.DetachVolume failed to start", "volume", attachedVolume)
 				}
 			}
 		}
 	}
 
-	rc.attachDesiredVolumes()
+	rc.attachDesiredVolumes(logger)
 
 	// Update Node Status
-	err := rc.nodeStatusUpdater.UpdateNodeStatuses()
+	err := rc.nodeStatusUpdater.UpdateNodeStatuses(logger)
 	if err != nil {
-		klog.Warningf("UpdateNodeStatuses failed with: %v", err)
+		logger.Info("UpdateNodeStatuses failed", "err", err)
 	}
 }
 
-func (rc *reconciler) attachDesiredVolumes() {
+func (rc *reconciler) attachDesiredVolumes(logger klog.Logger) {
 	// Ensure volumes that should be attached are attached.
 	for _, volumeToAttach := range rc.desiredStateOfWorld.GetVolumesToAttach() {
 		if util.IsMultiAttachAllowed(volumeToAttach.VolumeSpec) {
 			// Don't even try to start an operation if there is already one running for the given volume and node.
 			if rc.attacherDetacher.IsOperationPending(volumeToAttach.VolumeName, "" /* podName */, volumeToAttach.NodeName) {
-				klog.V(10).Infof("Operation for volume %q is already running for node %q. Can't start attach", volumeToAttach.VolumeName, volumeToAttach.NodeName)
+				logger.V(10).Info("Operation for volume is already running for node. Can't start attach", "node", klog.KRef("", string(volumeToAttach.NodeName)), "volumeName", volumeToAttach.VolumeName)
 				continue
 			}
 		} else {
 			// Don't even try to start an operation if there is already one running for the given volume
 			if rc.attacherDetacher.IsOperationPending(volumeToAttach.VolumeName, "" /* podName */, "" /* nodeName */) {
-				klog.V(10).Infof("Operation for volume %q is already running. Can't start attach for %q", volumeToAttach.VolumeName, volumeToAttach.NodeName)
+				logger.V(10).Info("Operation for volume is already running. Can't start attach for node", "node", klog.KRef("", string(volumeToAttach.NodeName)), "volumeNames", volumeToAttach.VolumeName)
 				continue
 			}
 		}
@@ -323,8 +325,8 @@ func (rc *reconciler) attachDesiredVolumes() {
 		attachState := rc.actualStateOfWorld.GetAttachState(volumeToAttach.VolumeName, volumeToAttach.NodeName)
 		if attachState == cache.AttachStateAttached {
 			// Volume/Node exists, touch it to reset detachRequestedTime
-			klog.V(5).InfoS("Volume attached--touching", "volume", volumeToAttach)
-			rc.actualStateOfWorld.ResetDetachRequestTime(volumeToAttach.VolumeName, volumeToAttach.NodeName)
+			logger.V(10).Info("Volume attached--touching", "volume", volumeToAttach)
+			rc.actualStateOfWorld.ResetDetachRequestTime(logger, volumeToAttach.VolumeName, volumeToAttach.NodeName)
 			continue
 		}
 
@@ -332,7 +334,7 @@ func (rc *reconciler) attachDesiredVolumes() {
 			nodes := rc.actualStateOfWorld.GetNodesForAttachedVolume(volumeToAttach.VolumeName)
 			if len(nodes) > 0 {
 				if !volumeToAttach.MultiAttachErrorReported {
-					rc.reportMultiAttachError(volumeToAttach, nodes)
+					rc.reportMultiAttachError(logger, volumeToAttach, nodes)
 					rc.desiredStateOfWorld.SetMultiAttachError(volumeToAttach.VolumeName, volumeToAttach.NodeName)
 				}
 				continue
@@ -340,22 +342,22 @@ func (rc *reconciler) attachDesiredVolumes() {
 		}
 
 		// Volume/Node doesn't exist, spawn a goroutine to attach it
-		klog.V(5).InfoS("Starting attacherDetacher.AttachVolume", "volume", volumeToAttach)
-		err := rc.attacherDetacher.AttachVolume(volumeToAttach.VolumeToAttach, rc.actualStateOfWorld)
+		logger.V(5).Info("Starting attacherDetacher.AttachVolume", "volume", volumeToAttach)
+		err := rc.attacherDetacher.AttachVolume(logger, volumeToAttach.VolumeToAttach, rc.actualStateOfWorld)
 		if err == nil {
-			klog.InfoS("attacherDetacher.AttachVolume started", "volume", volumeToAttach)
+			logger.Info("attacherDetacher.AttachVolume started", "volumeName", volumeToAttach.VolumeName, "nodeName", volumeToAttach.NodeName, "scheduledPods", klog.KObjSlice(volumeToAttach.ScheduledPods))
 		}
 		if err != nil && !exponentialbackoff.IsExponentialBackoff(err) {
 			// Ignore exponentialbackoff.IsExponentialBackoff errors, they are expected.
 			// Log all other errors.
-			klog.ErrorS(err, "attacherDetacher.AttachVolume failed to start", "volume", volumeToAttach)
+			logger.Error(err, "attacherDetacher.AttachVolume failed to start", "volumeName", volumeToAttach.VolumeName, "nodeName", volumeToAttach.NodeName, "scheduledPods", klog.KObjSlice(volumeToAttach.ScheduledPods))
 		}
 	}
 }
 
 // reportMultiAttachError sends events and logs situation that a volume that
 // should be attached to a node is already attached to different node(s).
-func (rc *reconciler) reportMultiAttachError(volumeToAttach cache.VolumeToAttach, nodes []types.NodeName) {
+func (rc *reconciler) reportMultiAttachError(logger klog.Logger, volumeToAttach cache.VolumeToAttach, nodes []types.NodeName) {
 	// Filter out the current node from list of nodes where the volume is
 	// attached.
 	// Some methods need []string, some other needs []NodeName, collect both.
@@ -373,7 +375,6 @@ func (rc *reconciler) reportMultiAttachError(volumeToAttach cache.VolumeToAttach
 
 	// Get list of pods that use the volume on the other nodes.
 	pods := rc.desiredStateOfWorld.GetVolumePodsOnNodes(otherNodes, volumeToAttach.VolumeName)
-
 	if len(pods) == 0 {
 		// We did not find any pods that requests the volume. The pod must have been deleted already.
 		simpleMsg, _ := volumeToAttach.GenerateMsg("Multi-Attach error", "Volume is already exclusively attached to one node and can't be attached to another")
@@ -381,7 +382,7 @@ func (rc *reconciler) reportMultiAttachError(volumeToAttach cache.VolumeToAttach
 			rc.recorder.Eventf(pod, v1.EventTypeWarning, kevents.FailedAttachVolume, simpleMsg)
 		}
 		// Log detailed message to system admin
-		klog.InfoS("Multi-Attach error: volume is already exclusively attached and can't be attached to another node", "attachedTo", otherNodesStr, "volume", volumeToAttach)
+		logger.Info("Multi-Attach error: volume is already exclusively attached and can't be attached to another node", "attachedTo", otherNodesStr, "volume", volumeToAttach)
 		return
 	}
 
@@ -417,5 +418,5 @@ func (rc *reconciler) reportMultiAttachError(volumeToAttach cache.VolumeToAttach
 	}
 
 	// Log all pods for system admin
-	klog.InfoS("Multi-Attach error: volume is already used by pods", "pods", klog.KObjSlice(pods), "attachedTo", otherNodesStr, "volume", volumeToAttach)
+	logger.Info("Multi-Attach error: volume is already used by pods", "pods", klog.KObjSlice(pods), "attachedTo", otherNodesStr, "volume", volumeToAttach)
 }
