@@ -60,7 +60,7 @@ func TestCheckpointStateRestore(t *testing.T) {
 			containermap.ContainerMap{},
 			"",
 			&stateMemory{
-				defaultCPUSet: cpuset.NewCPUSet(4, 5, 6),
+				defaultCPUSet: cpuset.New(4, 5, 6),
 			},
 		},
 		{
@@ -82,11 +82,11 @@ func TestCheckpointStateRestore(t *testing.T) {
 			&stateMemory{
 				assignments: ContainerCPUAssignments{
 					"pod": map[string]cpuset.CPUSet{
-						"container1": cpuset.NewCPUSet(4, 5, 6),
-						"container2": cpuset.NewCPUSet(1, 2, 3),
+						"container1": cpuset.New(4, 5, 6),
+						"container2": cpuset.New(1, 2, 3),
 					},
 				},
-				defaultCPUSet: cpuset.NewCPUSet(1, 2, 3),
+				defaultCPUSet: cpuset.New(1, 2, 3),
 			},
 		},
 		{
@@ -165,7 +165,7 @@ func TestCheckpointStateRestore(t *testing.T) {
 			containermap.ContainerMap{},
 			"",
 			&stateMemory{
-				defaultCPUSet: cpuset.NewCPUSet(1, 2, 3),
+				defaultCPUSet: cpuset.New(1, 2, 3),
 			},
 		},
 		{
@@ -190,11 +190,11 @@ func TestCheckpointStateRestore(t *testing.T) {
 			&stateMemory{
 				assignments: ContainerCPUAssignments{
 					"pod": map[string]cpuset.CPUSet{
-						"container1": cpuset.NewCPUSet(4, 5, 6),
-						"container2": cpuset.NewCPUSet(1, 2, 3),
+						"container1": cpuset.New(4, 5, 6),
+						"container2": cpuset.New(1, 2, 3),
 					},
 				},
-				defaultCPUSet: cpuset.NewCPUSet(1, 2, 3),
+				defaultCPUSet: cpuset.New(1, 2, 3),
 			},
 		},
 	}
@@ -242,14 +242,14 @@ func TestCheckpointStateStore(t *testing.T) {
 	}{
 		{
 			"Store default cpu set",
-			&stateMemory{defaultCPUSet: cpuset.NewCPUSet(1, 2, 3)},
+			&stateMemory{defaultCPUSet: cpuset.New(1, 2, 3)},
 		},
 		{
 			"Store assignments",
 			&stateMemory{
 				assignments: map[string]map[string]cpuset.CPUSet{
 					"pod": {
-						"container1": cpuset.NewCPUSet(1, 5, 8),
+						"container1": cpuset.New(1, 5, 8),
 					},
 				},
 			},
@@ -301,29 +301,29 @@ func TestCheckpointStateHelpers(t *testing.T) {
 	}{
 		{
 			description:   "One container",
-			defaultCPUset: cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7, 8),
+			defaultCPUset: cpuset.New(0, 1, 2, 3, 4, 5, 6, 7, 8),
 			assignments: map[string]map[string]cpuset.CPUSet{
 				"pod": {
-					"c1": cpuset.NewCPUSet(0, 1),
+					"c1": cpuset.New(0, 1),
 				},
 			},
 		},
 		{
 			description:   "Two containers",
-			defaultCPUset: cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7, 8),
+			defaultCPUset: cpuset.New(0, 1, 2, 3, 4, 5, 6, 7, 8),
 			assignments: map[string]map[string]cpuset.CPUSet{
 				"pod": {
-					"c1": cpuset.NewCPUSet(0, 1),
-					"c2": cpuset.NewCPUSet(2, 3, 4, 5),
+					"c1": cpuset.New(0, 1),
+					"c2": cpuset.New(2, 3, 4, 5),
 				},
 			},
 		},
 		{
 			description:   "Container without assigned cpus",
-			defaultCPUset: cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7, 8),
+			defaultCPUset: cpuset.New(0, 1, 2, 3, 4, 5, 6, 7, 8),
 			assignments: map[string]map[string]cpuset.CPUSet{
 				"pod": {
-					"c1": cpuset.NewCPUSet(),
+					"c1": cpuset.New(),
 				},
 			},
 		},
@@ -377,10 +377,10 @@ func TestCheckpointStateClear(t *testing.T) {
 	}{
 		{
 			"Valid state",
-			cpuset.NewCPUSet(1, 5, 10),
+			cpuset.New(1, 5, 10),
 			map[string]map[string]cpuset.CPUSet{
 				"pod": {
-					"container1": cpuset.NewCPUSet(1, 4),
+					"container1": cpuset.New(1, 4),
 				},
 			},
 		},
@@ -404,7 +404,7 @@ func TestCheckpointStateClear(t *testing.T) {
 			state.SetCPUAssignments(tc.assignments)
 
 			state.ClearState()
-			if !cpuset.NewCPUSet().Equals(state.GetDefaultCPUSet()) {
+			if !cpuset.New().Equals(state.GetDefaultCPUSet()) {
 				t.Fatal("cleared state with non-empty default cpu set")
 			}
 			for pod := range tc.assignments {
