@@ -52,8 +52,13 @@ func TestPullImageWithError(t *testing.T) {
 	_, fakeImageService, fakeManager, err := createTestRuntimeManager()
 	assert.NoError(t, err)
 
+	// trying to pull an image with an invalid name should return an error
+	imageRef, err := fakeManager.PullImage(ctx, kubecontainer.ImageSpec{Image: ":invalid"}, nil, nil)
+	assert.Error(t, err)
+	assert.Equal(t, "", imageRef)
+
 	fakeImageService.InjectError("PullImage", fmt.Errorf("test-error"))
-	imageRef, err := fakeManager.PullImage(ctx, kubecontainer.ImageSpec{Image: "busybox"}, nil, nil)
+	imageRef, err = fakeManager.PullImage(ctx, kubecontainer.ImageSpec{Image: "busybox"}, nil, nil)
 	assert.Error(t, err)
 	assert.Equal(t, "", imageRef)
 
@@ -81,30 +86,6 @@ func TestListImages(t *testing.T) {
 	assert.Equal(t, expected.List(), actual.List())
 }
 
-func TestListImagesPinnedField(t *testing.T) {
-	ctx := context.Background()
-	_, fakeImageService, fakeManager, err := createTestRuntimeManager()
-	assert.NoError(t, err)
-
-	imagesPinned := map[string]bool{
-		"1111": false,
-		"2222": true,
-		"3333": false,
-	}
-	imageList := []string{}
-	for image, pinned := range imagesPinned {
-		fakeImageService.SetFakeImagePinned(image, pinned)
-		imageList = append(imageList, image)
-	}
-	fakeImageService.SetFakeImages(imageList)
-
-	actualImages, err := fakeManager.ListImages(ctx)
-	assert.NoError(t, err)
-	for _, image := range actualImages {
-		assert.Equal(t, imagesPinned[image.ID], image.Pinned)
-	}
-}
-
 func TestListImagesWithError(t *testing.T) {
 	ctx := context.Background()
 	_, fakeImageService, fakeManager, err := createTestRuntimeManager()
@@ -296,6 +277,62 @@ func TestPullWithSecrets(t *testing.T) {
 	}
 }
 
+func TestPullWithSecretsWithError(t *testing.T) {
+	ctx := context.Background()
+
+	dockerCfg := map[string]map[string]map[string]string{
+		"auths": {
+			"index.docker.io/v1/": {
+				"email": "passed-email",
+				"auth":  "cGFzc2VkLXVzZXI6cGFzc2VkLXBhc3N3b3Jk",
+			},
+		},
+	}
+
+	dockerConfigJSON, err := json.Marshal(dockerCfg)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	for _, test := range []struct {
+		name              string
+		imageName         string
+		passedSecrets     []v1.Secret
+		shouldInjectError bool
+	}{
+		{
+			name:          "invalid docker secret",
+			imageName:     "ubuntu",
+			passedSecrets: []v1.Secret{{Type: v1.SecretTypeDockercfg, Data: map[string][]byte{v1.DockerConfigKey: []byte("invalid")}}},
+		},
+		{
+			name:      "secret provided, pull failed",
+			imageName: "ubuntu",
+			passedSecrets: []v1.Secret{
+				{Type: v1.SecretTypeDockerConfigJson, Data: map[string][]byte{v1.DockerConfigKey: dockerConfigJSON}},
+			},
+			shouldInjectError: true,
+		},
+	} {
+		t.Run(test.name, func(t *testing.T) {
+			_, fakeImageService, fakeManager, err := createTestRuntimeManager()
+			assert.NoError(t, err)
+
+			if test.shouldInjectError {
+				fakeImageService.InjectError("PullImage", fmt.Errorf("test-error"))
+			}
+
+			imageRef, err := fakeManager.PullImage(ctx, kubecontainer.ImageSpec{Image: test.imageName}, test.passedSecrets, nil)
+			assert.Error(t, err)
+			assert.Equal(t, "", imageRef)
+
+			images, err := fakeManager.ListImages(ctx)
+			assert.NoError(t, err)
+			assert.Equal(t, 0, len(images))
+		})
+	}
+}
+
 func TestPullThenListWithAnnotations(t *testing.T) {
 	ctx := context.Background()
 	_, _, fakeManager, err := createTestRuntimeManager()
