@@ -26,7 +26,6 @@ import (
 	"strings"
 	"time"
 
-	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/fields"
@@ -34,6 +33,7 @@ import (
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
 	utilnet "k8s.io/apimachinery/pkg/util/net"
+	utilvalidation "k8s.io/apimachinery/pkg/util/validation"
 	"k8s.io/apimachinery/pkg/util/validation/field"
 	"k8s.io/apiserver/pkg/registry/generic"
 	"k8s.io/apiserver/pkg/storage"
@@ -44,7 +44,7 @@ import (
 	podutil "k8s.io/kubernetes/pkg/api/pod"
 	api "k8s.io/kubernetes/pkg/apis/core"
 	"k8s.io/kubernetes/pkg/apis/core/helper/qos"
-	"k8s.io/kubernetes/pkg/apis/core/validation"
+	corevalidation "k8s.io/kubernetes/pkg/apis/core/validation"
 	"k8s.io/kubernetes/pkg/features"
 	"k8s.io/kubernetes/pkg/kubelet/client"
 	proxyutil "k8s.io/kubernetes/pkg/proxy/util"
@@ -88,7 +88,6 @@ func (podStrategy) PrepareForCreate(ctx context.Context, obj runtime.Object) {
 
 	podutil.DropDisabledPodFields(pod, nil)
 
-	applySeccompVersionSkew(pod)
 	applyWaitingForSchedulingGatesCondition(pod)
 }
 
@@ -98,6 +97,14 @@ func (podStrategy) PrepareForUpdate(ctx context.Context, obj, old runtime.Object
 	oldPod := old.(*api.Pod)
 	newPod.Status = oldPod.Status
 
+	if utilfeature.DefaultFeatureGate.Enabled(features.InPlacePodVerticalScaling) {
+		// With support for in-place pod resizing, container resources are now mutable.
+		// If container resources are updated with new resource requests values, a pod resize is
+		// desired. The status of this request is reflected by setting Resize field to "Proposed"
+		// as a signal to the caller that the request is being considered.
+		podutil.MarkPodProposedForResize(oldPod, newPod)
+	}
+
 	podutil.DropDisabledPodFields(newPod, oldPod)
 }
 
@@ -105,12 +112,18 @@ func (podStrategy) PrepareForUpdate(ctx context.Context, obj, old runtime.Object
 func (podStrategy) Validate(ctx context.Context, obj runtime.Object) field.ErrorList {
 	pod := obj.(*api.Pod)
 	opts := podutil.GetValidationOptionsFromPodSpecAndMeta(&pod.Spec, nil, &pod.ObjectMeta, nil)
-	return validation.ValidatePodCreate(pod, opts)
+	return corevalidation.ValidatePodCreate(pod, opts)
 }
 
 // WarningsOnCreate returns warnings for the creation of the given object.
 func (podStrategy) WarningsOnCreate(ctx context.Context, obj runtime.Object) []string {
-	return podutil.GetWarningsForPod(ctx, obj.(*api.Pod), nil)
+	newPod := obj.(*api.Pod)
+	var warnings []string
+	if msgs := utilvalidation.IsDNS1123Label(newPod.Name); len(msgs) != 0 {
+		warnings = append(warnings, fmt.Sprintf("metadata.name: this is used in the Pod's hostname, which can result in surprising behavior; a DNS label is recommended: %v", msgs))
+	}
+	warnings = append(warnings, podutil.GetWarningsForPod(ctx, newPod, nil)...)
+	return warnings
 }
 
 // Canonicalize normalizes the object after validation.
@@ -128,7 +141,7 @@ func (podStrategy) ValidateUpdate(ctx context.Context, obj, old runtime.Object)
 	pod := obj.(*api.Pod)
 	oldPod := old.(*api.Pod)
 	opts := podutil.GetValidationOptionsFromPodSpecAndMeta(&pod.Spec, &oldPod.Spec, &pod.ObjectMeta, &oldPod.ObjectMeta)
-	return validation.ValidatePodUpdate(obj.(*api.Pod), old.(*api.Pod), opts)
+	return corevalidation.ValidatePodUpdate(obj.(*api.Pod), old.(*api.Pod), opts)
 }
 
 // WarningsOnUpdate returns warnings for the given update.
@@ -213,7 +226,7 @@ func (podStatusStrategy) ValidateUpdate(ctx context.Context, obj, old runtime.Ob
 	oldPod := old.(*api.Pod)
 	opts := podutil.GetValidationOptionsFromPodSpecAndMeta(&pod.Spec, &oldPod.Spec, &pod.ObjectMeta, &oldPod.ObjectMeta)
 
-	return validation.ValidatePodStatusUpdate(obj.(*api.Pod), old.(*api.Pod), opts)
+	return corevalidation.ValidatePodStatusUpdate(obj.(*api.Pod), old.(*api.Pod), opts)
 }
 
 // WarningsOnUpdate returns warnings for the given update.
@@ -251,7 +264,7 @@ func (podEphemeralContainersStrategy) ValidateUpdate(ctx context.Context, obj, o
 	newPod := obj.(*api.Pod)
 	oldPod := old.(*api.Pod)
 	opts := podutil.GetValidationOptionsFromPodSpecAndMeta(&newPod.Spec, &oldPod.Spec, &newPod.ObjectMeta, &oldPod.ObjectMeta)
-	return validation.ValidatePodEphemeralContainersUpdate(newPod, oldPod, opts)
+	return corevalidation.ValidatePodEphemeralContainersUpdate(newPod, oldPod, opts)
 }
 
 // WarningsOnUpdate returns warnings for the given update.
@@ -667,86 +680,3 @@ func applyWaitingForSchedulingGatesCondition(pod *api.Pod) {
 		Message: "Scheduling is blocked due to non-empty scheduling gates",
 	})
 }
-
-// applySeccompVersionSkew implements the version skew behavior described in:
-// https://github.com/kubernetes/enhancements/tree/master/keps/sig-node/135-seccomp#version-skew-strategy
-// Note that we dropped copying the field to annotation synchronization in
-// v1.25 with the functional removal of the annotations.
-func applySeccompVersionSkew(pod *api.Pod) {
-	// get possible annotation and field
-	annotation, hasAnnotation := pod.Annotations[v1.SeccompPodAnnotationKey]
-	hasField := false
-
-	if pod.Spec.SecurityContext != nil && pod.Spec.SecurityContext.SeccompProfile != nil {
-		hasField = true
-	}
-
-	// sync field and annotation
-	if hasAnnotation && !hasField {
-		newField := seccompFieldForAnnotation(annotation)
-
-		if newField != nil {
-			if pod.Spec.SecurityContext == nil {
-				pod.Spec.SecurityContext = &api.PodSecurityContext{}
-			}
-			pod.Spec.SecurityContext.SeccompProfile = newField
-		}
-	}
-
-	// Handle the containers of the pod
-	podutil.VisitContainers(&pod.Spec, podutil.AllFeatureEnabledContainers(),
-		func(ctr *api.Container, _ podutil.ContainerType) bool {
-			// get possible annotation and field
-			key := api.SeccompContainerAnnotationKeyPrefix + ctr.Name
-			annotation, hasAnnotation := pod.Annotations[key]
-
-			hasField := false
-			if ctr.SecurityContext != nil && ctr.SecurityContext.SeccompProfile != nil {
-				hasField = true
-			}
-
-			// sync field and annotation
-			if hasAnnotation && !hasField {
-				newField := seccompFieldForAnnotation(annotation)
-
-				if newField != nil {
-					if ctr.SecurityContext == nil {
-						ctr.SecurityContext = &api.SecurityContext{}
-					}
-					ctr.SecurityContext.SeccompProfile = newField
-				}
-			}
-
-			return true
-		})
-}
-
-// seccompFieldForAnnotation takes a pod annotation and returns the converted
-// seccomp profile field.
-func seccompFieldForAnnotation(annotation string) *api.SeccompProfile {
-	// If only seccomp annotations are specified, copy the values into the
-	// corresponding fields. This ensures that existing applications continue
-	// to enforce seccomp, and prevents the kubelet from needing to resolve
-	// annotations & fields.
-	if annotation == v1.SeccompProfileNameUnconfined {
-		return &api.SeccompProfile{Type: api.SeccompProfileTypeUnconfined}
-	}
-
-	if annotation == api.SeccompProfileRuntimeDefault || annotation == api.DeprecatedSeccompProfileDockerDefault {
-		return &api.SeccompProfile{Type: api.SeccompProfileTypeRuntimeDefault}
-	}
-
-	if strings.HasPrefix(annotation, v1.SeccompLocalhostProfileNamePrefix) {
-		localhostProfile := strings.TrimPrefix(annotation, v1.SeccompLocalhostProfileNamePrefix)
-		if localhostProfile != "" {
-			return &api.SeccompProfile{
-				Type:             api.SeccompProfileTypeLocalhost,
-				LocalhostProfile: &localhostProfile,
-			}
-		}
-	}
-
-	// we can only reach this code path if the localhostProfile name has a zero
-	// length or if the annotation has an unrecognized value
-	return nil
-}
