@@ -42,18 +42,15 @@ func TestNodeAffinity(t *testing.T) {
 		wantPreFilterStatus *framework.Status
 		wantPreFilterResult *framework.PreFilterResult
 		args                config.NodeAffinityArgs
-		disablePreFilter    bool
+		runPreFilter        bool
 	}{
-		{
-			name: "no selector",
-			pod:  &v1.Pod{},
-		},
 		{
 			name: "missing labels",
 			pod: st.MakePod().NodeSelector(map[string]string{
 				"foo": "bar",
 			}).Obj(),
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "same labels",
@@ -63,6 +60,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "node labels are superset",
@@ -73,6 +71,7 @@ func TestNodeAffinity(t *testing.T) {
 				"foo": "bar",
 				"baz": "blah",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "node labels are subset",
@@ -83,7 +82,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchExpressions using In operator that matches the existing node",
@@ -111,6 +111,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchExpressions using Gt operator that matches the existing node",
@@ -139,6 +140,7 @@ func TestNodeAffinity(t *testing.T) {
 				// We use two digit to denote major version and two digit for minor version.
 				"kernel-version": "0206",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchExpressions using NotIn operator that matches the existing node",
@@ -166,6 +168,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"mem-type": "DDR3",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchExpressions using Exists operator that matches the existing node",
@@ -192,6 +195,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"GPU": "NVIDIA-GRID-K1",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with affinity that don't match node's labels won't schedule onto the node",
@@ -219,43 +223,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
-		},
-		{
-			name: "Pod with a nil []NodeSelectorTerm in affinity, can't match the node's labels and won't schedule onto the node",
-			pod: &v1.Pod{
-				Spec: v1.PodSpec{
-					Affinity: &v1.Affinity{
-						NodeAffinity: &v1.NodeAffinity{
-							RequiredDuringSchedulingIgnoredDuringExecution: &v1.NodeSelector{
-								NodeSelectorTerms: nil,
-							},
-						},
-					},
-				},
-			},
-			labels: map[string]string{
-				"foo": "bar",
-			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
-		},
-		{
-			name: "Pod with an empty []NodeSelectorTerm in affinity, can't match the node's labels and won't schedule onto the node",
-			pod: &v1.Pod{
-				Spec: v1.PodSpec{
-					Affinity: &v1.Affinity{
-						NodeAffinity: &v1.NodeAffinity{
-							RequiredDuringSchedulingIgnoredDuringExecution: &v1.NodeSelector{
-								NodeSelectorTerms: []v1.NodeSelectorTerm{},
-							},
-						},
-					},
-				},
-			},
-			labels: map[string]string{
-				"foo": "bar",
-			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with empty MatchExpressions is not a valid value will match no objects and won't schedule onto the node",
@@ -277,7 +246,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with no Affinity will schedule onto a node",
@@ -285,6 +255,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			wantPreFilterStatus: framework.NewStatus(framework.Skip),
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with Affinity but nil NodeSelector will schedule onto a node",
@@ -300,6 +272,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			wantPreFilterStatus: framework.NewStatus(framework.Skip),
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with multiple matchExpressions ANDed that matches the existing node",
@@ -330,6 +304,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"GPU": "NVIDIA-GRID-K1",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with multiple matchExpressions ANDed that doesn't match the existing node",
@@ -360,7 +335,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"GPU": "NVIDIA-GRID-K1",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with multiple NodeSelectorTerms ORed in affinity, matches the node's labels and will schedule onto the node",
@@ -397,6 +373,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with an Affinity and a PodSpec.NodeSelector(the old thing that we are deprecating) " +
@@ -427,6 +404,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with an Affinity matches node's labels but the PodSpec.NodeSelector(the old thing that we are deprecating) " +
@@ -457,7 +435,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "barrrrrr",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with an invalid value in Affinity term won't be scheduled onto the node",
@@ -485,7 +464,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchFields using In operator that matches the existing node",
@@ -512,6 +492,7 @@ func TestNodeAffinity(t *testing.T) {
 			},
 			nodeName:            "node1",
 			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.NewString("node1")},
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with matchFields using In operator that does not match the existing node",
@@ -539,6 +520,7 @@ func TestNodeAffinity(t *testing.T) {
 			nodeName:            "node2",
 			wantStatus:          framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
 			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.NewString("node1")},
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with two terms: matchFields does not match, but matchExpressions matches",
@@ -555,6 +537,11 @@ func TestNodeAffinity(t *testing.T) {
 												Operator: v1.NodeSelectorOpIn,
 												Values:   []string{"node1"},
 											},
+											{
+												Key:      metav1.ObjectNameField,
+												Operator: v1.NodeSelectorOpIn,
+												Values:   []string{"node2"},
+											},
 										},
 									},
 									{
@@ -572,8 +559,9 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName: "node2",
-			labels:   map[string]string{"foo": "bar"},
+			nodeName:     "node2",
+			labels:       map[string]string{"foo": "bar"},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with one term: matchFields does not match, but matchExpressions matches",
@@ -609,6 +597,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels:              map[string]string{"foo": "bar"},
 			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.NewString("node1")},
 			wantStatus:          framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with one term: both matchFields and matchExpressions match",
@@ -643,6 +632,7 @@ func TestNodeAffinity(t *testing.T) {
 			nodeName:            "node1",
 			labels:              map[string]string{"foo": "bar"},
 			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.NewString("node1")},
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with two terms: both matchFields and matchExpressions do not match",
@@ -676,9 +666,10 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName:   "node2",
-			labels:     map[string]string{"foo": "bar"},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			nodeName:     "node2",
+			labels:       map[string]string{"foo": "bar"},
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with two terms of node.Name affinity",
@@ -714,6 +705,7 @@ func TestNodeAffinity(t *testing.T) {
 			},
 			nodeName:            "node2",
 			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.NewString("node1", "node2")},
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with two conflicting mach field requirements",
@@ -747,6 +739,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels:              map[string]string{"foo": "bar"},
 			wantPreFilterStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, errReasonConflict),
 			wantStatus:          framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter:        true,
 		},
 		{
 			name: "Matches added affinity and Pod's node affinity",
@@ -786,6 +779,7 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Matches added affinity but not Pod's node affinity",
@@ -825,7 +819,8 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name:     "Doesn't match added affinity",
@@ -847,7 +842,8 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, errReasonEnforced),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, errReasonEnforced),
+			runPreFilter: true,
 		},
 		{
 			name: "Matches node selector correctly even if PreFilter is not called",
@@ -862,7 +858,7 @@ func TestNodeAffinity(t *testing.T) {
 				"foo": "bar",
 				"baz": "blah",
 			},
-			disablePreFilter: true,
+			runPreFilter: false,
 		},
 		{
 			name: "Matches node affinity correctly even if PreFilter is not called",
@@ -893,7 +889,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"GPU": "NVIDIA-GRID-K1",
 			},
-			disablePreFilter: true,
+			runPreFilter: false,
 		},
 	}
 
@@ -913,7 +909,7 @@ func TestNodeAffinity(t *testing.T) {
 
 			state := framework.NewCycleState()
 			var gotStatus *framework.Status
-			if !test.disablePreFilter {
+			if test.runPreFilter {
 				gotPreFilterResult, gotStatus := p.(framework.PreFilterPlugin).PreFilter(context.Background(), state, test.pod)
 				if diff := cmp.Diff(test.wantPreFilterStatus, gotStatus); diff != "" {
 					t.Errorf("unexpected PreFilter Status (-want,+got):\n%s", diff)
@@ -1006,12 +1002,12 @@ func TestNodeAffinityPriority(t *testing.T) {
 	}
 
 	tests := []struct {
-		name            string
-		pod             *v1.Pod
-		nodes           []*v1.Node
-		expectedList    framework.NodeScoreList
-		args            config.NodeAffinityArgs
-		disablePreScore bool
+		name         string
+		pod          *v1.Pod
+		nodes        []*v1.Node
+		expectedList framework.NodeScoreList
+		args         config.NodeAffinityArgs
+		runPreScore  bool
 	}{
 		{
 			name: "all nodes are same priority as NodeAffinity is nil",
@@ -1026,6 +1022,7 @@ func TestNodeAffinityPriority(t *testing.T) {
 				{ObjectMeta: metav1.ObjectMeta{Name: "node3", Labels: label3}},
 			},
 			expectedList: []framework.NodeScore{{Name: "node1", Score: 0}, {Name: "node2", Score: 0}, {Name: "node3", Score: 0}},
+			runPreScore:  true,
 		},
 		{
 			name: "no node matches preferred scheduling requirements in NodeAffinity of pod so all nodes' priority is zero",
@@ -1040,6 +1037,7 @@ func TestNodeAffinityPriority(t *testing.T) {
 				{ObjectMeta: metav1.ObjectMeta{Name: "node3", Labels: label3}},
 			},
 			expectedList: []framework.NodeScore{{Name: "node1", Score: 0}, {Name: "node2", Score: 0}, {Name: "node3", Score: 0}},
+			runPreScore:  true,
 		},
 		{
 			name: "only node1 matches the preferred scheduling requirements of pod",
@@ -1054,6 +1052,7 @@ func TestNodeAffinityPriority(t *testing.T) {
 				{ObjectMeta: metav1.ObjectMeta{Name: "node3", Labels: label3}},
 			},
 			expectedList: []framework.NodeScore{{Name: "node1", Score: framework.MaxNodeScore}, {Name: "node2", Score: 0}, {Name: "node3", Score: 0}},
+			runPreScore:  true,
 		},
 		{
 			name: "all nodes matches the preferred scheduling requirements of pod but with different priorities ",
@@ -1068,6 +1067,7 @@ func TestNodeAffinityPriority(t *testing.T) {
 				{ObjectMeta: metav1.ObjectMeta{Name: "node2", Labels: label2}},
 			},
 			expectedList: []framework.NodeScore{{Name: "node1", Score: 18}, {Name: "node5", Score: framework.MaxNodeScore}, {Name: "node2", Score: 36}},
+			runPreScore:  true,
 		},
 		{
 			name: "added affinity",
@@ -1080,6 +1080,7 @@ func TestNodeAffinityPriority(t *testing.T) {
 			args: config.NodeAffinityArgs{
 				AddedAffinity: affinity1.NodeAffinity,
 			},
+			runPreScore: true,
 		},
 		{
 			name: "added affinity and pod has default affinity",
@@ -1112,6 +1113,7 @@ func TestNodeAffinityPriority(t *testing.T) {
 					},
 				},
 			},
+			runPreScore: true,
 		},
 		{
 			name: "calculate the priorities correctly even if PreScore is not called",
@@ -1125,8 +1127,8 @@ func TestNodeAffinityPriority(t *testing.T) {
 				{ObjectMeta: metav1.ObjectMeta{Name: "node5", Labels: label5}},
 				{ObjectMeta: metav1.ObjectMeta{Name: "node2", Labels: label2}},
 			},
-			expectedList:    []framework.NodeScore{{Name: "node1", Score: 18}, {Name: "node5", Score: framework.MaxNodeScore}, {Name: "node2", Score: 36}},
-			disablePreScore: true,
+			expectedList: []framework.NodeScore{{Name: "node1", Score: 18}, {Name: "node5", Score: framework.MaxNodeScore}, {Name: "node2", Score: 36}},
+			runPreScore:  true,
 		},
 	}
 
@@ -1142,7 +1144,7 @@ func TestNodeAffinityPriority(t *testing.T) {
 				t.Fatalf("Creating plugin: %v", err)
 			}
 			var status *framework.Status
-			if !test.disablePreScore {
+			if test.runPreScore {
 				status = p.(framework.PreScorePlugin).PreScore(ctx, state, test.pod, test.nodes)
 				if !status.IsSuccess() {
 					t.Errorf("unexpected error: %v", status)
