@@ -91,7 +91,6 @@ type F struct {
 	G []int             `json:"fg"`
 	H []bool            `json:"fh"`
 	I []float32         `json:"fi"`
-	J []byte            `json:"fj"`
 }
 
 type G struct {
@@ -342,7 +341,7 @@ func TestRoundTrip(t *testing.T) {
 
 // TestUnknownFields checks for the collection of unknown
 // field errors from the various possible locations of
-// unknown fields (e.g. fields on struct, inlined sturct, slice, etc)
+// unknown fields (e.g. fields on struct, inlined struct, slice, etc)
 func TestUnknownFields(t *testing.T) {
 	// simples checks that basic unknown fields are found
 	// in fields, subfields and slices.
@@ -752,10 +751,6 @@ func TestUnrecognized(t *testing.T) {
 			data: "{\"ff\":[\"abc\"],\"fg\":[123],\"fh\":[true,false]}",
 			obj:  &F{},
 		},
-		{
-			data: "{\"fj\":\"\"}",
-			obj:  &F{},
-		},
 		{
 			// Invalid string data
 			data: "{\"fa\":123}",
