@@ -17,6 +17,7 @@ limitations under the License.
 package options
 
 import (
+	"context"
 	"fmt"
 	"net/http"
 	"strconv"
@@ -33,7 +34,7 @@ import (
 	"k8s.io/apiserver/pkg/server"
 	"k8s.io/apiserver/pkg/server/healthz"
 	"k8s.io/apiserver/pkg/server/options/encryptionconfig"
-	kmsconfigcontroller "k8s.io/apiserver/pkg/server/options/encryptionconfig/controller"
+	encryptionconfigcontroller "k8s.io/apiserver/pkg/server/options/encryptionconfig/controller"
 	serverstorage "k8s.io/apiserver/pkg/server/storage"
 	"k8s.io/apiserver/pkg/storage/storagebackend"
 	storagefactory "k8s.io/apiserver/pkg/storage/storagebackend/factory"
@@ -228,10 +229,11 @@ func (s *EtcdOptions) Complete(
 	}
 
 	if len(s.EncryptionProviderConfigFilepath) != 0 {
-		ctxTransformers, closeTransformers := wait.ContextForChannel(stopCh)
-		ctxServer, _ := wait.ContextForChannel(stopCh) // explicitly ignore cancel here because we do not own the server's lifecycle
+		ctxServer := wait.ContextForChannel(stopCh)
+		// nolint:govet // The only code path where closeTransformers does not get called is when it gets stored in dynamicTransformers.
+		ctxTransformers, closeTransformers := context.WithCancel(ctxServer)
 
-		encryptionConfiguration, err := encryptionconfig.LoadEncryptionConfig(s.EncryptionProviderConfigFilepath, s.EncryptionProviderConfigAutomaticReload, ctxTransformers.Done())
+		encryptionConfiguration, err := encryptionconfig.LoadEncryptionConfig(ctxTransformers, s.EncryptionProviderConfigFilepath, s.EncryptionProviderConfigAutomaticReload)
 		if err != nil {
 			// in case of error, we want to close partially initialized (if any) transformers
 			closeTransformers()
@@ -247,25 +249,22 @@ func (s *EtcdOptions) Complete(
 				return fmt.Errorf("failed to start kms encryption config hot reload controller. only 1 health check should be available when reload is enabled")
 			}
 
+			// Here the dynamic transformers take ownership of the transformers and their cancellation.
 			dynamicTransformers := encryptionconfig.NewDynamicTransformers(encryptionConfiguration.Transformers, encryptionConfiguration.HealthChecks[0], closeTransformers, encryptionConfiguration.KMSCloseGracePeriod)
 
-			s.resourceTransformers = dynamicTransformers
-			s.kmsPluginHealthzChecks = []healthz.HealthChecker{dynamicTransformers}
-
 			// add post start hook to start hot reload controller
 			// adding this hook here will ensure that it gets configured exactly once
 			err = addPostStartHook(
 				"start-encryption-provider-config-automatic-reload",
-				func(hookContext server.PostStartHookContext) error {
-					kmsConfigController := kmsconfigcontroller.NewDynamicKMSEncryptionConfiguration(
-						"kms-encryption-config",
+				func(_ server.PostStartHookContext) error {
+					dynamicEncryptionConfigController := encryptionconfigcontroller.NewDynamicEncryptionConfiguration(
+						"encryption-provider-config-automatic-reload-controller",
 						s.EncryptionProviderConfigFilepath,
 						dynamicTransformers,
 						encryptionConfiguration.EncryptionFileContentHash,
-						ctxServer.Done(),
 					)
 
-					go kmsConfigController.Run(ctxServer)
+					go dynamicEncryptionConfigController.Run(ctxServer)
 
 					return nil
 				},
@@ -275,6 +274,9 @@ func (s *EtcdOptions) Complete(
 				closeTransformers()
 				return fmt.Errorf("failed to add post start hook for kms encryption config hot reload controller: %w", err)
 			}
+
+			s.resourceTransformers = dynamicTransformers
+			s.kmsPluginHealthzChecks = []healthz.HealthChecker{dynamicTransformers}
 		} else {
 			s.resourceTransformers = encryptionconfig.StaticTransformers(encryptionConfiguration.Transformers)
 			s.kmsPluginHealthzChecks = encryptionConfiguration.HealthChecks
@@ -285,6 +287,7 @@ func (s *EtcdOptions) Complete(
 
 	s.complete = true
 
+	// nolint:govet // The only code path where closeTransformers does not get called is when it gets stored in dynamicTransformers.
 	return nil
 }
 
@@ -441,10 +444,6 @@ func (s *SimpleStorageFactory) ResourcePrefix(resource schema.GroupResource) str
 	return resource.Group + "/" + resource.Resource
 }
 
-func (s *SimpleStorageFactory) Configs() []storagebackend.Config {
-	return serverstorage.Configs(s.StorageConfig)
-}
-
 func (s *SimpleStorageFactory) Backends() []serverstorage.Backend {
 	// nothing should ever call this method but we still provide a functional implementation
 	return serverstorage.Backends(s.StorageConfig)
@@ -475,10 +474,6 @@ func (t *transformerStorageFactory) ResourcePrefix(resource schema.GroupResource
 	return t.delegate.ResourcePrefix(resource)
 }
 
-func (t *transformerStorageFactory) Configs() []storagebackend.Config {
-	return t.delegate.Configs()
-}
-
 func (t *transformerStorageFactory) Backends() []serverstorage.Backend {
 	return t.delegate.Backends()
 }
