@@ -24,8 +24,10 @@ import (
 	"k8s.io/apimachinery/pkg/api/meta"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/cli-runtime/pkg/genericclioptions"
+	openapiclient "k8s.io/client-go/openapi"
 	cmdutil "k8s.io/kubectl/pkg/cmd/util"
 	"k8s.io/kubectl/pkg/explain"
+	openapiv3explain "k8s.io/kubectl/pkg/explain/v2"
 	"k8s.io/kubectl/pkg/util/i18n"
 	"k8s.io/kubectl/pkg/util/openapi"
 	"k8s.io/kubectl/pkg/util/templates"
@@ -49,6 +51,9 @@ var (
 
 		# Get the documentation of a specific field of a resource
 		kubectl explain pods.spec.containers`))
+
+	plaintextTemplateName          = "plaintext"
+	plaintextOpenAPIV2TemplateName = "plaintext-openapiv2"
 )
 
 type ExplainOptions struct {
@@ -62,12 +67,20 @@ type ExplainOptions struct {
 
 	Mapper meta.RESTMapper
 	Schema openapi.Resources
+
+	// Name of the template to use with the openapiv3 template renderer. If
+	// `EnableOpenAPIV3` is disabled, this does nothing
+	OutputFormat string
+
+	// Client capable of fetching openapi documents from the user's cluster
+	OpenAPIV3Client openapiclient.Client
 }
 
 func NewExplainOptions(parent string, streams genericclioptions.IOStreams) *ExplainOptions {
 	return &ExplainOptions{
-		IOStreams: streams,
-		CmdParent: parent,
+		IOStreams:    streams,
+		CmdParent:    parent,
+		OutputFormat: plaintextTemplateName,
 	}
 }
 
@@ -89,6 +102,10 @@ func NewCmdExplain(parent string, f cmdutil.Factory, streams genericclioptions.I
 	}
 	cmd.Flags().BoolVar(&o.Recursive, "recursive", o.Recursive, "Print the fields of fields (Currently only 1 level deep)")
 	cmd.Flags().StringVar(&o.APIVersion, "api-version", o.APIVersion, "Get different explanations for particular API version (API group/version)")
+
+	// Only enable --output as a valid flag if the feature is enabled
+	cmd.Flags().StringVar(&o.OutputFormat, "output", plaintextTemplateName, "Format in which to render the schema (plaintext, plaintext-openapiv2)")
+
 	return cmd
 }
 
@@ -104,6 +121,12 @@ func (o *ExplainOptions) Complete(f cmdutil.Factory, cmd *cobra.Command, args []
 		return err
 	}
 
+	// Only openapi v3 needs the discovery client.
+	o.OpenAPIV3Client, err = f.OpenAPIV3Client()
+	if err != nil {
+		return err
+	}
+
 	o.args = args
 	return nil
 }
@@ -121,13 +144,10 @@ func (o *ExplainOptions) Validate() error {
 
 // Run executes the appropriate steps to print a model's documentation
 func (o *ExplainOptions) Run() error {
-	recursive := o.Recursive
-	apiVersionString := o.APIVersion
-
 	var fullySpecifiedGVR schema.GroupVersionResource
 	var fieldsPath []string
 	var err error
-	if len(apiVersionString) == 0 {
+	if len(o.APIVersion) == 0 {
 		fullySpecifiedGVR, fieldsPath, err = explain.SplitAndParseResourceRequestWithMatchingPrefix(o.args[0], o.Mapper)
 		if err != nil {
 			return err
@@ -142,6 +162,45 @@ func (o *ExplainOptions) Run() error {
 		}
 	}
 
+	// Fallback to openapiv2 implementation using special template name
+	switch o.OutputFormat {
+	case plaintextOpenAPIV2TemplateName:
+		return o.renderOpenAPIV2(fullySpecifiedGVR, fieldsPath)
+	case plaintextTemplateName:
+		// Check whether the server reponds to OpenAPIV3.
+		if _, err := o.OpenAPIV3Client.Paths(); err != nil {
+			// Use v2 renderer if server does not support v3
+			return o.renderOpenAPIV2(fullySpecifiedGVR, fieldsPath)
+		}
+
+		fallthrough
+	default:
+		if len(o.APIVersion) > 0 {
+			apiVersion, err := schema.ParseGroupVersion(o.APIVersion)
+			if err != nil {
+				return err
+			}
+			fullySpecifiedGVR.Group = apiVersion.Group
+			fullySpecifiedGVR.Version = apiVersion.Version
+		}
+
+		return openapiv3explain.PrintModelDescription(
+			fieldsPath,
+			o.Out,
+			o.OpenAPIV3Client,
+			fullySpecifiedGVR,
+			o.Recursive,
+			o.OutputFormat,
+		)
+	}
+}
+
+func (o *ExplainOptions) renderOpenAPIV2(
+	fullySpecifiedGVR schema.GroupVersionResource,
+	fieldsPath []string,
+) error {
+	var err error
+
 	gvk, _ := o.Mapper.KindFor(fullySpecifiedGVR)
 	if gvk.Empty() {
 		gvk, err = o.Mapper.KindFor(fullySpecifiedGVR.GroupResource().WithVersion(""))
@@ -150,8 +209,8 @@ func (o *ExplainOptions) Run() error {
 		}
 	}
 
-	if len(apiVersionString) != 0 {
-		apiVersion, err := schema.ParseGroupVersion(apiVersionString)
+	if len(o.APIVersion) != 0 {
+		apiVersion, err := schema.ParseGroupVersion(o.APIVersion)
 		if err != nil {
 			return err
 		}
@@ -163,5 +222,5 @@ func (o *ExplainOptions) Run() error {
 		return fmt.Errorf("couldn't find resource for %q", gvk)
 	}
 
-	return explain.PrintModelDescription(fieldsPath, o.Out, schema, gvk, recursive)
+	return explain.PrintModelDescription(fieldsPath, o.Out, schema, gvk, o.Recursive)
 }
