@@ -18,6 +18,7 @@ package rollout
 
 import (
 	"fmt"
+	"sort"
 
 	"github.com/spf13/cobra"
 
@@ -153,6 +154,44 @@ func (o *RolloutHistoryOptions) Run() error {
 		return err
 	}
 
+	if o.PrintFlags.OutputFlagSpecified() {
+		printer, err := o.PrintFlags.ToPrinter()
+		if err != nil {
+			return err
+		}
+
+		return r.Visit(func(info *resource.Info, err error) error {
+			if err != nil {
+				return err
+			}
+
+			mapping := info.ResourceMapping()
+			historyViewer, err := o.HistoryViewer(o.RESTClientGetter, mapping)
+			if err != nil {
+				return err
+			}
+			historyInfo, err := historyViewer.GetHistory(info.Namespace, info.Name)
+			if err != nil {
+				return err
+			}
+
+			if o.Revision > 0 {
+				printer.PrintObj(historyInfo[o.Revision], o.Out)
+			} else {
+				sortedKeys := make([]int64, 0, len(historyInfo))
+				for k := range historyInfo {
+					sortedKeys = append(sortedKeys, k)
+				}
+				sort.Slice(sortedKeys, func(i, j int) bool { return sortedKeys[i] < sortedKeys[j] })
+				for _, k := range sortedKeys {
+					printer.PrintObj(historyInfo[k], o.Out)
+				}
+			}
+
+			return nil
+		})
+	}
+
 	return r.Visit(func(info *resource.Info, err error) error {
 		if err != nil {
 			return err
