@@ -17,6 +17,7 @@ limitations under the License.
 package apimachinery
 
 import (
+	"context"
 	"regexp"
 
 	"k8s.io/apimachinery/pkg/version"
@@ -36,7 +37,7 @@ var _ = SIGDescribe("server version", func() {
 	   Description: Ensure that an API server version can be retrieved.
 	   Both the major and minor versions MUST only be an integer.
 	*/
-	framework.ConformanceIt("should find the server version", func() {
+	framework.ConformanceIt("should find the server version", func(ctx context.Context) {
 
 		ginkgo.By("Request ServerVersion")
 
