@@ -53,28 +53,28 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 			e2eskipper.SkipIfNodeOSDistroIs("windows")
 		})
 
-		ginkgo.It("new files should be created with FSGroup ownership when container is root", func() {
-			doTestSetgidFSGroup(f, 0, v1.StorageMediumMemory)
+		ginkgo.It("new files should be created with FSGroup ownership when container is root", func(ctx context.Context) {
+			doTestSetgidFSGroup(ctx, f, 0, v1.StorageMediumMemory)
 		})
 
-		ginkgo.It("new files should be created with FSGroup ownership when container is non-root", func() {
-			doTestSetgidFSGroup(f, nonRootUID, v1.StorageMediumMemory)
+		ginkgo.It("new files should be created with FSGroup ownership when container is non-root", func(ctx context.Context) {
+			doTestSetgidFSGroup(ctx, f, nonRootUID, v1.StorageMediumMemory)
 		})
 
-		ginkgo.It("nonexistent volume subPath should have the correct mode and owner using FSGroup", func() {
-			doTestSubPathFSGroup(f, nonRootUID, v1.StorageMediumMemory)
+		ginkgo.It("nonexistent volume subPath should have the correct mode and owner using FSGroup", func(ctx context.Context) {
+			doTestSubPathFSGroup(ctx, f, nonRootUID, v1.StorageMediumMemory)
 		})
 
-		ginkgo.It("files with FSGroup ownership should support (root,0644,tmpfs)", func() {
-			doTest0644FSGroup(f, 0, v1.StorageMediumMemory)
+		ginkgo.It("files with FSGroup ownership should support (root,0644,tmpfs)", func(ctx context.Context) {
+			doTest0644FSGroup(ctx, f, 0, v1.StorageMediumMemory)
 		})
 
-		ginkgo.It("volume on default medium should have the correct mode using FSGroup", func() {
-			doTestVolumeModeFSGroup(f, 0, v1.StorageMediumDefault)
+		ginkgo.It("volume on default medium should have the correct mode using FSGroup", func(ctx context.Context) {
+			doTestVolumeModeFSGroup(ctx, f, 0, v1.StorageMediumDefault)
 		})
 
-		ginkgo.It("volume on tmpfs should have the correct mode using FSGroup", func() {
-			doTestVolumeModeFSGroup(f, 0, v1.StorageMediumMemory)
+		ginkgo.It("volume on tmpfs should have the correct mode using FSGroup", func(ctx context.Context) {
+			doTestVolumeModeFSGroup(ctx, f, 0, v1.StorageMediumMemory)
 		})
 	})
 
@@ -84,8 +84,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume and 'medium' as 'Memory', the volume MUST have mode set as -rwxrwxrwx and mount type set to tmpfs.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or the medium = 'Memory'.
 	*/
-	framework.ConformanceIt("volume on tmpfs should have the correct mode [LinuxOnly] [NodeConformance]", func() {
-		doTestVolumeMode(f, 0, v1.StorageMediumMemory)
+	framework.ConformanceIt("volume on tmpfs should have the correct mode [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTestVolumeMode(ctx, f, 0, v1.StorageMediumMemory)
 	})
 
 	/*
@@ -94,8 +94,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume and 'medium' as 'Memory', the volume mode set to 0644. The volume MUST have mode -rw-r--r-- and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID, or the medium = 'Memory'.
 	*/
-	framework.ConformanceIt("should support (root,0644,tmpfs) [LinuxOnly] [NodeConformance]", func() {
-		doTest0644(f, 0, v1.StorageMediumMemory)
+	framework.ConformanceIt("should support (root,0644,tmpfs) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0644(ctx, f, 0, v1.StorageMediumMemory)
 	})
 
 	/*
@@ -104,8 +104,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume and 'medium' as 'Memory', the volume mode set to 0666. The volume MUST have mode -rw-rw-rw- and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID, or the medium = 'Memory'.
 	*/
-	framework.ConformanceIt("should support (root,0666,tmpfs) [LinuxOnly] [NodeConformance]", func() {
-		doTest0666(f, 0, v1.StorageMediumMemory)
+	framework.ConformanceIt("should support (root,0666,tmpfs) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0666(ctx, f, 0, v1.StorageMediumMemory)
 	})
 
 	/*
@@ -114,8 +114,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume and 'medium' as 'Memory', the volume mode set to 0777.  The volume MUST have mode set as -rwxrwxrwx and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID, or the medium = 'Memory'.
 	*/
-	framework.ConformanceIt("should support (root,0777,tmpfs) [LinuxOnly] [NodeConformance]", func() {
-		doTest0777(f, 0, v1.StorageMediumMemory)
+	framework.ConformanceIt("should support (root,0777,tmpfs) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0777(ctx, f, 0, v1.StorageMediumMemory)
 	})
 
 	/*
@@ -124,8 +124,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume and 'medium' as 'Memory', the volume mode set to 0644. Volume is mounted into the container where container is run as a non-root user. The volume MUST have mode -rw-r--r-- and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID, or the medium = 'Memory'.
 	*/
-	framework.ConformanceIt("should support (non-root,0644,tmpfs) [LinuxOnly] [NodeConformance]", func() {
-		doTest0644(f, nonRootUID, v1.StorageMediumMemory)
+	framework.ConformanceIt("should support (non-root,0644,tmpfs) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0644(ctx, f, nonRootUID, v1.StorageMediumMemory)
 	})
 
 	/*
@@ -134,8 +134,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume and 'medium' as 'Memory', the volume mode set to 0666. Volume is mounted into the container where container is run as a non-root user. The volume MUST have mode -rw-rw-rw- and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID, or the medium = 'Memory'.
 	*/
-	framework.ConformanceIt("should support (non-root,0666,tmpfs) [LinuxOnly] [NodeConformance]", func() {
-		doTest0666(f, nonRootUID, v1.StorageMediumMemory)
+	framework.ConformanceIt("should support (non-root,0666,tmpfs) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0666(ctx, f, nonRootUID, v1.StorageMediumMemory)
 	})
 
 	/*
@@ -144,8 +144,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume and 'medium' as 'Memory', the volume mode set to 0777. Volume is mounted into the container where container is run as a non-root user. The volume MUST have mode -rwxrwxrwx and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID, or the medium = 'Memory'.
 	*/
-	framework.ConformanceIt("should support (non-root,0777,tmpfs) [LinuxOnly] [NodeConformance]", func() {
-		doTest0777(f, nonRootUID, v1.StorageMediumMemory)
+	framework.ConformanceIt("should support (non-root,0777,tmpfs) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0777(ctx, f, nonRootUID, v1.StorageMediumMemory)
 	})
 
 	/*
@@ -154,8 +154,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume, the volume MUST have mode set as -rwxrwxrwx and mount type set to tmpfs.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions.
 	*/
-	framework.ConformanceIt("volume on default medium should have the correct mode [LinuxOnly] [NodeConformance]", func() {
-		doTestVolumeMode(f, 0, v1.StorageMediumDefault)
+	framework.ConformanceIt("volume on default medium should have the correct mode [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTestVolumeMode(ctx, f, 0, v1.StorageMediumDefault)
 	})
 
 	/*
@@ -164,8 +164,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume, the volume mode set to 0644. The volume MUST have mode -rw-r--r-- and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID.
 	*/
-	framework.ConformanceIt("should support (root,0644,default) [LinuxOnly] [NodeConformance]", func() {
-		doTest0644(f, 0, v1.StorageMediumDefault)
+	framework.ConformanceIt("should support (root,0644,default) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0644(ctx, f, 0, v1.StorageMediumDefault)
 	})
 
 	/*
@@ -174,8 +174,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume, the volume mode set to 0666. The volume MUST have mode -rw-rw-rw- and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID.
 	*/
-	framework.ConformanceIt("should support (root,0666,default) [LinuxOnly] [NodeConformance]", func() {
-		doTest0666(f, 0, v1.StorageMediumDefault)
+	framework.ConformanceIt("should support (root,0666,default) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0666(ctx, f, 0, v1.StorageMediumDefault)
 	})
 
 	/*
@@ -184,8 +184,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume, the volume mode set to 0777.  The volume MUST have mode set as -rwxrwxrwx and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID.
 	*/
-	framework.ConformanceIt("should support (root,0777,default) [LinuxOnly] [NodeConformance]", func() {
-		doTest0777(f, 0, v1.StorageMediumDefault)
+	framework.ConformanceIt("should support (root,0777,default) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0777(ctx, f, 0, v1.StorageMediumDefault)
 	})
 
 	/*
@@ -194,8 +194,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume, the volume mode set to 0644. Volume is mounted into the container where container is run as a non-root user. The volume MUST have mode -rw-r--r-- and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID.
 	*/
-	framework.ConformanceIt("should support (non-root,0644,default) [LinuxOnly] [NodeConformance]", func() {
-		doTest0644(f, nonRootUID, v1.StorageMediumDefault)
+	framework.ConformanceIt("should support (non-root,0644,default) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0644(ctx, f, nonRootUID, v1.StorageMediumDefault)
 	})
 
 	/*
@@ -204,8 +204,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume, the volume mode set to 0666. Volume is mounted into the container where container is run as a non-root user. The volume MUST have mode -rw-rw-rw- and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID.
 	*/
-	framework.ConformanceIt("should support (non-root,0666,default) [LinuxOnly] [NodeConformance]", func() {
-		doTest0666(f, nonRootUID, v1.StorageMediumDefault)
+	framework.ConformanceIt("should support (non-root,0666,default) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0666(ctx, f, nonRootUID, v1.StorageMediumDefault)
 	})
 
 	/*
@@ -214,8 +214,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume, the volume mode set to 0777. Volume is mounted into the container where container is run as a non-root user. The volume MUST have mode -rwxrwxrwx and mount type set to tmpfs and the contents MUST be readable.
 		This test is marked LinuxOnly since Windows does not support setting specific file permissions, or running as UID / GID.
 	*/
-	framework.ConformanceIt("should support (non-root,0777,default) [LinuxOnly] [NodeConformance]", func() {
-		doTest0777(f, nonRootUID, v1.StorageMediumDefault)
+	framework.ConformanceIt("should support (non-root,0777,default) [LinuxOnly] [NodeConformance]", func(ctx context.Context) {
+		doTest0777(ctx, f, nonRootUID, v1.StorageMediumDefault)
 	})
 
 	/*
@@ -224,7 +224,7 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Description: A Pod created with an 'emptyDir' Volume, should share volumes between the containeres in the pod. The two busybox image containers should share the volumes mounted to the pod.
 		The main container should wait until the sub container drops a file, and main container access the shared data.
 	*/
-	framework.ConformanceIt("pod should support shared volumes between containers", func() {
+	framework.ConformanceIt("pod should support shared volumes between containers", func(ctx context.Context) {
 		var (
 			volumeName                 = "shared-data"
 			busyBoxMainVolumeMountPath = "/usr/share/volumeshare"
@@ -283,8 +283,8 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		}
 
 		ginkgo.By("Creating Pod")
-		e2epod.NewPodClient(f).Create(pod)
-		e2epod.WaitForPodNameRunningInNamespace(f.ClientSet, pod.Name, f.Namespace.Name)
+		e2epod.NewPodClient(f).Create(ctx, pod)
+		framework.ExpectNoError(e2epod.WaitForPodNameRunningInNamespace(ctx, f.ClientSet, pod.Name, f.Namespace.Name))
 
 		ginkgo.By("Reading file content from the nginx-container")
 		result := e2epod.ExecShellInContainer(f, pod.Name, busyBoxMainContainerName, fmt.Sprintf("cat %s", busyBoxMainVolumeFilePath))
@@ -296,7 +296,7 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 		Testname: EmptyDir, Memory backed volume is sized to specified limit
 		Description: A Pod created with an 'emptyDir' Volume backed by memory should be sized to user provided value.
 	*/
-	ginkgo.It("pod should support memory backed volumes of specified size", func() {
+	ginkgo.It("pod should support memory backed volumes of specified size", func(ctx context.Context) {
 		var (
 			volumeName                 = "shared-data"
 			busyBoxMainVolumeMountPath = "/usr/share/volumeshare"
@@ -343,14 +343,14 @@ var _ = SIGDescribe("EmptyDir volumes", func() {
 
 		var err error
 		ginkgo.By("Creating Pod")
-		pod = e2epod.NewPodClient(f).CreateSync(pod)
+		pod = e2epod.NewPodClient(f).CreateSync(ctx, pod)
 
 		ginkgo.By("Waiting for the pod running")
-		err = e2epod.WaitForPodNameRunningInNamespace(f.ClientSet, pod.Name, f.Namespace.Name)
+		err = e2epod.WaitForPodNameRunningInNamespace(ctx, f.ClientSet, pod.Name, f.Namespace.Name)
 		framework.ExpectNoError(err, "failed to deploy pod %s", pod.Name)
 
 		ginkgo.By("Getting the pod")
-		pod, err = e2epod.NewPodClient(f).Get(context.TODO(), pod.Name, metav1.GetOptions{})
+		pod, err = e2epod.NewPodClient(f).Get(ctx, pod.Name, metav1.GetOptions{})
 		framework.ExpectNoError(err, "failed to get pod %s", pod.Name)
 
 		ginkgo.By("Reading empty dir size")
@@ -364,7 +364,7 @@ const (
 	volumeName    = "test-volume"
 )
 
-func doTestSetgidFSGroup(f *framework.Framework, uid int64, medium v1.StorageMedium) {
+func doTestSetgidFSGroup(ctx context.Context, f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	var (
 		filePath = path.Join(volumePath, "test-file")
 		source   = &v1.EmptyDirVolumeSource{Medium: medium}
@@ -391,10 +391,10 @@ func doTestSetgidFSGroup(f *framework.Framework, uid int64, medium v1.StorageMed
 	if medium == v1.StorageMediumMemory {
 		out = append(out, "mount type of \"/test-volume\": tmpfs")
 	}
-	e2epodoutput.TestContainerOutput(f, msg, pod, 0, out)
+	e2epodoutput.TestContainerOutput(ctx, f, msg, pod, 0, out)
 }
 
-func doTestSubPathFSGroup(f *framework.Framework, uid int64, medium v1.StorageMedium) {
+func doTestSubPathFSGroup(ctx context.Context, f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	var (
 		subPath = "test-sub"
 		source  = &v1.EmptyDirVolumeSource{Medium: medium}
@@ -424,10 +424,10 @@ func doTestSubPathFSGroup(f *framework.Framework, uid int64, medium v1.StorageMe
 	if medium == v1.StorageMediumMemory {
 		out = append(out, "mount type of \"/test-volume\": tmpfs")
 	}
-	e2epodoutput.TestContainerOutput(f, msg, pod, 0, out)
+	e2epodoutput.TestContainerOutput(ctx, f, msg, pod, 0, out)
 }
 
-func doTestVolumeModeFSGroup(f *framework.Framework, uid int64, medium v1.StorageMedium) {
+func doTestVolumeModeFSGroup(ctx context.Context, f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	var (
 		source = &v1.EmptyDirVolumeSource{Medium: medium}
 		pod    = testPodWithVolume(uid, volumePath, source)
@@ -449,10 +449,10 @@ func doTestVolumeModeFSGroup(f *framework.Framework, uid int64, medium v1.Storag
 	if medium == v1.StorageMediumMemory {
 		out = append(out, "mount type of \"/test-volume\": tmpfs")
 	}
-	e2epodoutput.TestContainerOutput(f, msg, pod, 0, out)
+	e2epodoutput.TestContainerOutput(ctx, f, msg, pod, 0, out)
 }
 
-func doTest0644FSGroup(f *framework.Framework, uid int64, medium v1.StorageMedium) {
+func doTest0644FSGroup(ctx context.Context, f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	var (
 		filePath = path.Join(volumePath, "test-file")
 		source   = &v1.EmptyDirVolumeSource{Medium: medium}
@@ -477,10 +477,10 @@ func doTest0644FSGroup(f *framework.Framework, uid int64, medium v1.StorageMediu
 	if medium == v1.StorageMediumMemory {
 		out = append(out, "mount type of \"/test-volume\": tmpfs")
 	}
-	e2epodoutput.TestContainerOutput(f, msg, pod, 0, out)
+	e2epodoutput.TestContainerOutput(ctx, f, msg, pod, 0, out)
 }
 
-func doTestVolumeMode(f *framework.Framework, uid int64, medium v1.StorageMedium) {
+func doTestVolumeMode(ctx context.Context, f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	var (
 		source = &v1.EmptyDirVolumeSource{Medium: medium}
 		pod    = testPodWithVolume(uid, volumePath, source)
@@ -499,10 +499,10 @@ func doTestVolumeMode(f *framework.Framework, uid int64, medium v1.StorageMedium
 	if medium == v1.StorageMediumMemory {
 		out = append(out, "mount type of \"/test-volume\": tmpfs")
 	}
-	e2epodoutput.TestContainerOutput(f, msg, pod, 0, out)
+	e2epodoutput.TestContainerOutput(ctx, f, msg, pod, 0, out)
 }
 
-func doTest0644(f *framework.Framework, uid int64, medium v1.StorageMedium) {
+func doTest0644(ctx context.Context, f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	var (
 		filePath = path.Join(volumePath, "test-file")
 		source   = &v1.EmptyDirVolumeSource{Medium: medium}
@@ -524,10 +524,10 @@ func doTest0644(f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	if medium == v1.StorageMediumMemory {
 		out = append(out, "mount type of \"/test-volume\": tmpfs")
 	}
-	e2epodoutput.TestContainerOutput(f, msg, pod, 0, out)
+	e2epodoutput.TestContainerOutput(ctx, f, msg, pod, 0, out)
 }
 
-func doTest0666(f *framework.Framework, uid int64, medium v1.StorageMedium) {
+func doTest0666(ctx context.Context, f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	var (
 		filePath = path.Join(volumePath, "test-file")
 		source   = &v1.EmptyDirVolumeSource{Medium: medium}
@@ -549,10 +549,10 @@ func doTest0666(f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	if medium == v1.StorageMediumMemory {
 		out = append(out, "mount type of \"/test-volume\": tmpfs")
 	}
-	e2epodoutput.TestContainerOutput(f, msg, pod, 0, out)
+	e2epodoutput.TestContainerOutput(ctx, f, msg, pod, 0, out)
 }
 
-func doTest0777(f *framework.Framework, uid int64, medium v1.StorageMedium) {
+func doTest0777(ctx context.Context, f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	var (
 		filePath = path.Join(volumePath, "test-file")
 		source   = &v1.EmptyDirVolumeSource{Medium: medium}
@@ -574,7 +574,7 @@ func doTest0777(f *framework.Framework, uid int64, medium v1.StorageMedium) {
 	if medium == v1.StorageMediumMemory {
 		out = append(out, "mount type of \"/test-volume\": tmpfs")
 	}
-	e2epodoutput.TestContainerOutput(f, msg, pod, 0, out)
+	e2epodoutput.TestContainerOutput(ctx, f, msg, pod, 0, out)
 }
 
 func formatMedium(medium v1.StorageMedium) string {
