@@ -51,24 +51,24 @@ var _ = SIGDescribe("[Feature:Example]", func() {
 
 	var c clientset.Interface
 	var ns string
-	ginkgo.BeforeEach(func() {
+	ginkgo.BeforeEach(func(ctx context.Context) {
 		c = f.ClientSet
 		ns = f.Namespace.Name
 
 		// this test wants powerful permissions.  Since the namespace names are unique, we can leave this
 		// lying around so we don't have to race any caches
-		err := e2eauth.BindClusterRoleInNamespace(c.RbacV1(), "edit", f.Namespace.Name,
+		err := e2eauth.BindClusterRoleInNamespace(ctx, c.RbacV1(), "edit", f.Namespace.Name,
 			rbacv1.Subject{Kind: rbacv1.ServiceAccountKind, Namespace: f.Namespace.Name, Name: "default"})
 		framework.ExpectNoError(err)
 
-		err = e2eauth.WaitForAuthorizationUpdate(c.AuthorizationV1(),
+		err = e2eauth.WaitForAuthorizationUpdate(ctx, c.AuthorizationV1(),
 			serviceaccount.MakeUsername(f.Namespace.Name, "default"),
 			f.Namespace.Name, "create", schema.GroupResource{Resource: "pods"}, true)
 		framework.ExpectNoError(err)
 	})
 
 	ginkgo.Describe("Liveness", func() {
-		ginkgo.It("liveness pods should be automatically restarted", func() {
+		ginkgo.It("liveness pods should be automatically restarted", func(ctx context.Context) {
 			test := "test/fixtures/doc-yaml/user-guide/liveness"
 			execYaml := readFile(test, "exec-liveness.yaml.in")
 			httpYaml := readFile(test, "http-liveness.yaml.in")
@@ -80,10 +80,10 @@ var _ = SIGDescribe("[Feature:Example]", func() {
 			var wg sync.WaitGroup
 			passed := true
 			checkRestart := func(podName string, timeout time.Duration) {
-				err := e2epod.WaitForPodNameRunningInNamespace(c, podName, ns)
+				err := e2epod.WaitForPodNameRunningInNamespace(ctx, c, podName, ns)
 				framework.ExpectNoError(err)
 				for t := time.Now(); time.Since(t) < timeout; time.Sleep(framework.Poll) {
-					pod, err := c.CoreV1().Pods(ns).Get(context.TODO(), podName, metav1.GetOptions{})
+					pod, err := c.CoreV1().Pods(ns).Get(ctx, podName, metav1.GetOptions{})
 					framework.ExpectNoError(err, fmt.Sprintf("getting pod %s", podName))
 					stat := podutil.GetExistingContainerStatus(pod.Status.ContainerStatuses, podName)
 					framework.Logf("Pod: %s, restart count:%d", stat.Name, stat.RestartCount)
@@ -115,7 +115,7 @@ var _ = SIGDescribe("[Feature:Example]", func() {
 	})
 
 	ginkgo.Describe("Secret", func() {
-		ginkgo.It("should create a pod that reads a secret", func() {
+		ginkgo.It("should create a pod that reads a secret", func(ctx context.Context) {
 			test := "test/fixtures/doc-yaml/user-guide/secrets"
 			secretYaml := readFile(test, "secret.yaml")
 			podYaml := readFile(test, "secret-pod.yaml.in")
@@ -125,7 +125,7 @@ var _ = SIGDescribe("[Feature:Example]", func() {
 			ginkgo.By("creating secret and pod")
 			e2ekubectl.RunKubectlOrDieInput(ns, secretYaml, "create", "-f", "-")
 			e2ekubectl.RunKubectlOrDieInput(ns, podYaml, "create", "-f", "-")
-			err := e2epod.WaitForPodNoLongerRunningInNamespace(c, podName, ns)
+			err := e2epod.WaitForPodNoLongerRunningInNamespace(ctx, c, podName, ns)
 			framework.ExpectNoError(err)
 
 			ginkgo.By("checking if secret was read correctly")
@@ -135,14 +135,14 @@ var _ = SIGDescribe("[Feature:Example]", func() {
 	})
 
 	ginkgo.Describe("Downward API", func() {
-		ginkgo.It("should create a pod that prints his name and namespace", func() {
+		ginkgo.It("should create a pod that prints his name and namespace", func(ctx context.Context) {
 			test := "test/fixtures/doc-yaml/user-guide/downward-api"
 			podYaml := readFile(test, "dapi-pod.yaml.in")
 			podName := "dapi-test-pod"
 
 			ginkgo.By("creating the pod")
 			e2ekubectl.RunKubectlOrDieInput(ns, podYaml, "create", "-f", "-")
-			err := e2epod.WaitForPodNoLongerRunningInNamespace(c, podName, ns)
+			err := e2epod.WaitForPodNoLongerRunningInNamespace(ctx, c, podName, ns)
 			framework.ExpectNoError(err)
 
 			ginkgo.By("checking if name and namespace were passed correctly")
