@@ -89,17 +89,17 @@ func makeCudaAdditionDevicePluginTestPod() *v1.Pod {
 	return testPod
 }
 
-func logOSImages(f *framework.Framework) {
-	nodeList, err := f.ClientSet.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{})
+func logOSImages(ctx context.Context, f *framework.Framework) {
+	nodeList, err := f.ClientSet.CoreV1().Nodes().List(ctx, metav1.ListOptions{})
 	framework.ExpectNoError(err, "getting node list")
 	for _, node := range nodeList.Items {
 		framework.Logf("Nodename: %v, OS Image: %v", node.Name, node.Status.NodeInfo.OSImage)
 	}
 }
 
-func areGPUsAvailableOnAllSchedulableNodes(f *framework.Framework) bool {
+func areGPUsAvailableOnAllSchedulableNodes(ctx context.Context, f *framework.Framework) bool {
 	framework.Logf("Getting list of Nodes from API server")
-	nodeList, err := f.ClientSet.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{})
+	nodeList, err := f.ClientSet.CoreV1().Nodes().List(ctx, metav1.ListOptions{})
 	framework.ExpectNoError(err, "getting node list")
 	for _, node := range nodeList.Items {
 		if node.Spec.Unschedulable {
@@ -115,8 +115,8 @@ func areGPUsAvailableOnAllSchedulableNodes(f *framework.Framework) bool {
 	return true
 }
 
-func getGPUsAvailable(f *framework.Framework) int64 {
-	nodeList, err := f.ClientSet.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{})
+func getGPUsAvailable(ctx context.Context, f *framework.Framework) int64 {
+	nodeList, err := f.ClientSet.CoreV1().Nodes().List(ctx, metav1.ListOptions{})
 	framework.ExpectNoError(err, "getting node list")
 	var gpusAvailable int64
 	for _, node := range nodeList.Items {
@@ -128,8 +128,8 @@ func getGPUsAvailable(f *framework.Framework) int64 {
 }
 
 // SetupNVIDIAGPUNode install Nvidia Drivers and wait for Nvidia GPUs to be available on nodes
-func SetupNVIDIAGPUNode(f *framework.Framework, setupResourceGatherer bool) *e2edebug.ContainerResourceGatherer {
-	logOSImages(f)
+func SetupNVIDIAGPUNode(ctx context.Context, f *framework.Framework, setupResourceGatherer bool) *e2edebug.ContainerResourceGatherer {
+	logOSImages(ctx, f)
 
 	var err error
 	var ds *appsv1.DaemonSet
@@ -137,7 +137,7 @@ func SetupNVIDIAGPUNode(f *framework.Framework, setupResourceGatherer bool) *e2e
 	if dsYamlURLFromEnv != "" {
 		// Using DaemonSet from remote URL
 		framework.Logf("Using remote nvidia-driver-installer daemonset manifest from %v", dsYamlURLFromEnv)
-		ds, err = e2emanifest.DaemonSetFromURL(dsYamlURLFromEnv)
+		ds, err = e2emanifest.DaemonSetFromURL(ctx, dsYamlURLFromEnv)
 		framework.ExpectNoError(err, "failed get remote")
 	} else {
 		// Using default local DaemonSet
@@ -149,14 +149,14 @@ func SetupNVIDIAGPUNode(f *framework.Framework, setupResourceGatherer bool) *e2e
 	}
 	gpuResourceName = e2egpu.NVIDIAGPUResourceName
 	ds.Namespace = f.Namespace.Name
-	_, err = f.ClientSet.AppsV1().DaemonSets(f.Namespace.Name).Create(context.TODO(), ds, metav1.CreateOptions{})
+	_, err = f.ClientSet.AppsV1().DaemonSets(f.Namespace.Name).Create(ctx, ds, metav1.CreateOptions{})
 	framework.ExpectNoError(err, "failed to create nvidia-driver-installer daemonset")
 	framework.Logf("Successfully created daemonset to install Nvidia drivers.")
 
-	pods, err := e2eresource.WaitForControlledPods(f.ClientSet, ds.Namespace, ds.Name, extensionsinternal.Kind("DaemonSet"))
+	pods, err := e2eresource.WaitForControlledPods(ctx, f.ClientSet, ds.Namespace, ds.Name, extensionsinternal.Kind("DaemonSet"))
 	framework.ExpectNoError(err, "failed to get pods controlled by the nvidia-driver-installer daemonset")
 
-	devicepluginPods, err := e2eresource.WaitForControlledPods(f.ClientSet, "kube-system", "nvidia-gpu-device-plugin", extensionsinternal.Kind("DaemonSet"))
+	devicepluginPods, err := e2eresource.WaitForControlledPods(ctx, f.ClientSet, "kube-system", "nvidia-gpu-device-plugin", extensionsinternal.Kind("DaemonSet"))
 	if err == nil {
 		framework.Logf("Adding deviceplugin addon pod.")
 		pods.Items = append(pods.Items, devicepluginPods.Items...)
@@ -165,15 +165,15 @@ func SetupNVIDIAGPUNode(f *framework.Framework, setupResourceGatherer bool) *e2e
 	var rsgather *e2edebug.ContainerResourceGatherer
 	if setupResourceGatherer {
 		framework.Logf("Starting ResourceUsageGather for the created DaemonSet pods.")
-		rsgather, err = e2edebug.NewResourceUsageGatherer(f.ClientSet, e2edebug.ResourceGathererOptions{InKubemark: false, Nodes: e2edebug.AllNodes, ResourceDataGatheringPeriod: 2 * time.Second, ProbeDuration: 2 * time.Second, PrintVerboseLogs: true}, pods)
+		rsgather, err = e2edebug.NewResourceUsageGatherer(ctx, f.ClientSet, e2edebug.ResourceGathererOptions{InKubemark: false, Nodes: e2edebug.AllNodes, ResourceDataGatheringPeriod: 2 * time.Second, ProbeDuration: 2 * time.Second, PrintVerboseLogs: true}, pods)
 		framework.ExpectNoError(err, "creating ResourceUsageGather for the daemonset pods")
-		go rsgather.StartGatheringData()
+		go rsgather.StartGatheringData(ctx)
 	}
 
 	// Wait for Nvidia GPUs to be available on nodes
 	framework.Logf("Waiting for drivers to be installed and GPUs to be available in Node Capacity...")
-	gomega.Eventually(func() bool {
-		return areGPUsAvailableOnAllSchedulableNodes(f)
+	gomega.Eventually(ctx, func(ctx context.Context) bool {
+		return areGPUsAvailableOnAllSchedulableNodes(ctx, f)
 	}, driverInstallTimeout, time.Second).Should(gomega.BeTrue())
 
 	return rsgather
@@ -190,19 +190,19 @@ func getGPUsPerPod() int64 {
 	return gpusPerPod
 }
 
-func testNvidiaGPUs(f *framework.Framework) {
-	rsgather := SetupNVIDIAGPUNode(f, true)
-	gpuPodNum := getGPUsAvailable(f) / getGPUsPerPod()
+func testNvidiaGPUs(ctx context.Context, f *framework.Framework) {
+	rsgather := SetupNVIDIAGPUNode(ctx, f, true)
+	gpuPodNum := getGPUsAvailable(ctx, f) / getGPUsPerPod()
 	framework.Logf("Creating %d pods and have the pods run a CUDA app", gpuPodNum)
 	podList := []*v1.Pod{}
 	for i := int64(0); i < gpuPodNum; i++ {
-		podList = append(podList, e2epod.NewPodClient(f).Create(makeCudaAdditionDevicePluginTestPod()))
+		podList = append(podList, e2epod.NewPodClient(f).Create(ctx, makeCudaAdditionDevicePluginTestPod()))
 	}
 	framework.Logf("Wait for all test pods to succeed")
 	// Wait for all pods to succeed
 	for _, pod := range podList {
-		e2epod.NewPodClient(f).WaitForSuccess(pod.Name, 5*time.Minute)
-		logContainers(f, pod)
+		e2epod.NewPodClient(f).WaitForSuccess(ctx, pod.Name, 5*time.Minute)
+		logContainers(ctx, f, pod)
 	}
 
 	framework.Logf("Stopping ResourceUsageGather")
@@ -213,9 +213,9 @@ func testNvidiaGPUs(f *framework.Framework) {
 	framework.ExpectNoError(err, "getting resource usage summary")
 }
 
-func logContainers(f *framework.Framework, pod *v1.Pod) {
+func logContainers(ctx context.Context, f *framework.Framework, pod *v1.Pod) {
 	for _, container := range pod.Spec.Containers {
-		logs, err := e2epod.GetPodLogs(f.ClientSet, f.Namespace.Name, pod.Name, container.Name)
+		logs, err := e2epod.GetPodLogs(ctx, f.ClientSet, f.Namespace.Name, pod.Name, container.Name)
 		framework.ExpectNoError(err, "Should be able to get container logs for container: %s", container.Name)
 		framework.Logf("Got container logs for %s:\n%v", container.Name, logs)
 	}
@@ -224,28 +224,28 @@ func logContainers(f *framework.Framework, pod *v1.Pod) {
 var _ = SIGDescribe("[Feature:GPUDevicePlugin]", func() {
 	f := framework.NewDefaultFramework("device-plugin-gpus")
 	f.NamespacePodSecurityEnforceLevel = admissionapi.LevelPrivileged
-	ginkgo.It("run Nvidia GPU Device Plugin tests", func() {
-		testNvidiaGPUs(f)
+	ginkgo.It("run Nvidia GPU Device Plugin tests", func(ctx context.Context) {
+		testNvidiaGPUs(ctx, f)
 	})
 })
 
-func testNvidiaGPUsJob(f *framework.Framework) {
-	_ = SetupNVIDIAGPUNode(f, false)
+func testNvidiaGPUsJob(ctx context.Context, f *framework.Framework) {
+	_ = SetupNVIDIAGPUNode(ctx, f, false)
 	// Job set to have 5 completions with parallelism of 1 to ensure that it lasts long enough to experience the node recreation
 	completions := int32(5)
 	ginkgo.By("Starting GPU job")
-	StartJob(f, completions)
+	StartJob(ctx, f, completions)
 
-	job, err := e2ejob.GetJob(f.ClientSet, f.Namespace.Name, "cuda-add")
+	job, err := e2ejob.GetJob(ctx, f.ClientSet, f.Namespace.Name, "cuda-add")
 	framework.ExpectNoError(err)
 
 	// make sure job is running by waiting for its first pod to start running
-	err = e2ejob.WaitForJobPodsRunning(f.ClientSet, f.Namespace.Name, job.Name, 1)
+	err = e2ejob.WaitForJobPodsRunning(ctx, f.ClientSet, f.Namespace.Name, job.Name, 1)
 	framework.ExpectNoError(err)
 
-	numNodes, err := e2enode.TotalRegistered(f.ClientSet)
+	numNodes, err := e2enode.TotalRegistered(ctx, f.ClientSet)
 	framework.ExpectNoError(err)
-	nodes, err := e2enode.CheckReady(f.ClientSet, numNodes, framework.NodeReadyInitialTimeout)
+	nodes, err := e2enode.CheckReady(ctx, f.ClientSet, numNodes, framework.NodeReadyInitialTimeout)
 	framework.ExpectNoError(err)
 
 	ginkgo.By("Recreating nodes")
@@ -254,17 +254,17 @@ func testNvidiaGPUsJob(f *framework.Framework) {
 	ginkgo.By("Done recreating nodes")
 
 	ginkgo.By("Waiting for gpu job to finish")
-	err = e2ejob.WaitForJobFinish(f.ClientSet, f.Namespace.Name, job.Name)
+	err = e2ejob.WaitForJobFinish(ctx, f.ClientSet, f.Namespace.Name, job.Name)
 	framework.ExpectNoError(err)
 	ginkgo.By("Done with gpu job")
 
 	gomega.Expect(job.Status.Failed).To(gomega.BeZero(), "Job pods failed during node recreation: %v", job.Status.Failed)
 
-	VerifyJobNCompletions(f, completions)
+	VerifyJobNCompletions(ctx, f, completions)
 }
 
 // StartJob starts a simple CUDA job that requests gpu and the specified number of completions
-func StartJob(f *framework.Framework, completions int32) {
+func StartJob(ctx context.Context, f *framework.Framework, completions int32) {
 	var activeSeconds int64 = 3600
 	testJob := e2ejob.NewTestJob("succeed", "cuda-add", v1.RestartPolicyAlways, 1, completions, &activeSeconds, 6)
 	testJob.Spec.Template.Spec = v1.PodSpec{
@@ -283,15 +283,15 @@ func StartJob(f *framework.Framework, completions int32) {
 		},
 	}
 	ns := f.Namespace.Name
-	_, err := e2ejob.CreateJob(f.ClientSet, ns, testJob)
+	_, err := e2ejob.CreateJob(ctx, f.ClientSet, ns, testJob)
 	framework.ExpectNoError(err)
 	framework.Logf("Created job %v", testJob)
 }
 
 // VerifyJobNCompletions verifies that the job has completions number of successful pods
-func VerifyJobNCompletions(f *framework.Framework, completions int32) {
+func VerifyJobNCompletions(ctx context.Context, f *framework.Framework, completions int32) {
 	ns := f.Namespace.Name
-	pods, err := e2ejob.GetJobPods(f.ClientSet, f.Namespace.Name, "cuda-add")
+	pods, err := e2ejob.GetJobPods(ctx, f.ClientSet, f.Namespace.Name, "cuda-add")
 	framework.ExpectNoError(err)
 	createdPods := pods.Items
 	createdPodNames := podNames(createdPods)
@@ -300,8 +300,8 @@ func VerifyJobNCompletions(f *framework.Framework, completions int32) {
 	successes := int32(0)
 	regex := regexp.MustCompile("PASSED")
 	for _, podName := range createdPodNames {
-		e2epod.NewPodClient(f).WaitForFinish(podName, 5*time.Minute)
-		logs, err := e2epod.GetPodLogs(f.ClientSet, ns, podName, "vector-addition")
+		e2epod.NewPodClient(f).WaitForFinish(ctx, podName, 5*time.Minute)
+		logs, err := e2epod.GetPodLogs(ctx, f.ClientSet, ns, podName, "vector-addition")
 		framework.ExpectNoError(err, "Should be able to get logs for pod %v", podName)
 		if regex.MatchString(logs) {
 			successes++
@@ -326,7 +326,7 @@ var _ = SIGDescribe("GPUDevicePluginAcrossRecreate [Feature:Recreate]", func() {
 	})
 	f := framework.NewDefaultFramework("device-plugin-gpus-recreate")
 	f.NamespacePodSecurityEnforceLevel = admissionapi.LevelPrivileged
-	ginkgo.It("run Nvidia GPU Device Plugin tests with a recreation", func() {
-		testNvidiaGPUsJob(f)
+	ginkgo.It("run Nvidia GPU Device Plugin tests with a recreation", func(ctx context.Context) {
+		testNvidiaGPUsJob(ctx, f)
 	})
 })
