@@ -139,17 +139,18 @@ var _ storageframework.EphemeralTestDriver = &hostpathCSIDriver{}
 // InitHostPathCSIDriver returns hostpathCSIDriver that implements TestDriver interface
 func InitHostPathCSIDriver() storageframework.TestDriver {
 	capabilities := map[storageframework.Capability]bool{
-		storageframework.CapPersistence:         true,
-		storageframework.CapSnapshotDataSource:  true,
-		storageframework.CapMultiPODs:           true,
-		storageframework.CapBlock:               true,
-		storageframework.CapPVCDataSource:       true,
-		storageframework.CapControllerExpansion: true,
-		storageframework.CapOfflineExpansion:    true,
-		storageframework.CapOnlineExpansion:     true,
-		storageframework.CapSingleNodeVolume:    true,
-		storageframework.CapReadWriteOncePod:    true,
-		storageframework.CapMultiplePVsSameID:   true,
+		storageframework.CapPersistence:                    true,
+		storageframework.CapSnapshotDataSource:             true,
+		storageframework.CapMultiPODs:                      true,
+		storageframework.CapBlock:                          true,
+		storageframework.CapPVCDataSource:                  true,
+		storageframework.CapControllerExpansion:            true,
+		storageframework.CapOfflineExpansion:               true,
+		storageframework.CapOnlineExpansion:                true,
+		storageframework.CapSingleNodeVolume:               true,
+		storageframework.CapReadWriteOncePod:               true,
+		storageframework.CapMultiplePVsSameID:              true,
+		storageframework.CapFSResizeFromSourceNotSupported: true,
 
 		// This is needed for the
 		// testsuites/volumelimits.go `should support volume limits`
@@ -184,7 +185,7 @@ func (h *hostpathCSIDriver) SkipUnsupportedTest(pattern storageframework.TestPat
 	}
 }
 
-func (h *hostpathCSIDriver) GetDynamicProvisionStorageClass(config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
+func (h *hostpathCSIDriver) GetDynamicProvisionStorageClass(ctx context.Context, config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
 	provisioner := config.GetUniqueDriverName()
 	parameters := map[string]string{}
 	ns := config.Framework.Namespace.Name
@@ -200,25 +201,25 @@ func (h *hostpathCSIDriver) GetCSIDriverName(config *storageframework.PerTestCon
 	return config.GetUniqueDriverName()
 }
 
-func (h *hostpathCSIDriver) GetSnapshotClass(config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
+func (h *hostpathCSIDriver) GetSnapshotClass(ctx context.Context, config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
 	snapshotter := config.GetUniqueDriverName()
 	ns := config.Framework.Namespace.Name
 
 	return utils.GenerateSnapshotClassSpec(snapshotter, parameters, ns)
 }
 
-func (h *hostpathCSIDriver) PrepareTest(f *framework.Framework) *storageframework.PerTestConfig {
+func (h *hostpathCSIDriver) PrepareTest(ctx context.Context, f *framework.Framework) *storageframework.PerTestConfig {
 	// Create secondary namespace which will be used for creating driver
-	driverNamespace := utils.CreateDriverNamespace(f)
+	driverNamespace := utils.CreateDriverNamespace(ctx, f)
 	driverns := driverNamespace.Name
 	testns := f.Namespace.Name
 
 	ginkgo.By(fmt.Sprintf("deploying %s driver", h.driverInfo.Name))
-	cancelLogging := utils.StartPodLogs(f, driverNamespace)
+	cancelLogging := utils.StartPodLogs(ctx, f, driverNamespace)
 	cs := f.ClientSet
 
 	// The hostpath CSI driver only works when everything runs on the same node.
-	node, err := e2enode.GetRandomReadySchedulableNode(cs)
+	node, err := e2enode.GetRandomReadySchedulableNode(ctx, cs)
 	framework.ExpectNoError(err)
 	config := &storageframework.PerTestConfig{
 		Driver:              h,
@@ -246,7 +247,7 @@ func (h *hostpathCSIDriver) PrepareTest(f *framework.Framework) *storageframewor
 		NodeName:                 node.Name,
 	}
 
-	cleanup, err := utils.CreateFromManifests(config.Framework, driverNamespace, func(item interface{}) error {
+	err = utils.CreateFromManifests(ctx, config.Framework, driverNamespace, func(item interface{}) error {
 		if err := utils.PatchCSIDeployment(config.Framework, o, item); err != nil {
 			return err
 		}
@@ -284,7 +285,6 @@ func (h *hostpathCSIDriver) PrepareTest(f *framework.Framework) *storageframewor
 		h.driverInfo.Name,
 		testns,
 		driverns,
-		cleanup,
 		cancelLogging)
 	ginkgo.DeferCleanup(cleanupFunc)
 
@@ -293,23 +293,24 @@ func (h *hostpathCSIDriver) PrepareTest(f *framework.Framework) *storageframewor
 
 // mockCSI
 type mockCSIDriver struct {
-	driverInfo             storageframework.DriverInfo
-	manifests              []string
-	podInfo                *bool
-	storageCapacity        *bool
-	attachable             bool
-	attachLimit            int
-	enableTopology         bool
-	enableNodeExpansion    bool
-	hooks                  Hooks
-	tokenRequests          []storagev1.TokenRequest
-	requiresRepublish      *bool
-	fsGroupPolicy          *storagev1.FSGroupPolicy
-	enableVolumeMountGroup bool
-	embedded               bool
-	calls                  MockCSICalls
-	embeddedCSIDriver      *mockdriver.CSIDriver
-	enableSELinuxMount     *bool
+	driverInfo                    storageframework.DriverInfo
+	manifests                     []string
+	podInfo                       *bool
+	storageCapacity               *bool
+	attachable                    bool
+	attachLimit                   int
+	enableTopology                bool
+	enableNodeExpansion           bool
+	hooks                         Hooks
+	tokenRequests                 []storagev1.TokenRequest
+	requiresRepublish             *bool
+	fsGroupPolicy                 *storagev1.FSGroupPolicy
+	enableVolumeMountGroup        bool
+	embedded                      bool
+	calls                         MockCSICalls
+	embeddedCSIDriver             *mockdriver.CSIDriver
+	enableSELinuxMount            *bool
+	enableRecoverExpansionFailure bool
 
 	// Additional values set during PrepareTest
 	clientSet       clientset.Interface
@@ -338,25 +339,26 @@ type MockCSITestDriver interface {
 
 	// GetCalls returns all currently observed gRPC calls. Only valid
 	// after PrepareTest.
-	GetCalls() ([]MockCSICall, error)
+	GetCalls(ctx context.Context) ([]MockCSICall, error)
 }
 
 // CSIMockDriverOpts defines options used for csi driver
 type CSIMockDriverOpts struct {
-	RegisterDriver         bool
-	DisableAttach          bool
-	PodInfo                *bool
-	StorageCapacity        *bool
-	AttachLimit            int
-	EnableTopology         bool
-	EnableResizing         bool
-	EnableNodeExpansion    bool
-	EnableSnapshot         bool
-	EnableVolumeMountGroup bool
-	TokenRequests          []storagev1.TokenRequest
-	RequiresRepublish      *bool
-	FSGroupPolicy          *storagev1.FSGroupPolicy
-	EnableSELinuxMount     *bool
+	RegisterDriver                bool
+	DisableAttach                 bool
+	PodInfo                       *bool
+	StorageCapacity               *bool
+	AttachLimit                   int
+	EnableTopology                bool
+	EnableResizing                bool
+	EnableNodeExpansion           bool
+	EnableSnapshot                bool
+	EnableVolumeMountGroup        bool
+	TokenRequests                 []storagev1.TokenRequest
+	RequiresRepublish             *bool
+	FSGroupPolicy                 *storagev1.FSGroupPolicy
+	EnableSELinuxMount            *bool
+	EnableRecoverExpansionFailure bool
 
 	// Embedded defines whether the CSI mock driver runs
 	// inside the cluster (false, the default) or just a proxy
@@ -377,6 +379,7 @@ type MockCSICall struct {
 	Method  string
 	Request struct {
 		VolumeContext map[string]string `json:"volume_context"`
+		Secrets       map[string]string `json:"secrets"`
 	}
 	FullError struct {
 		Code    codes.Code `json:"code"`
@@ -498,20 +501,21 @@ func InitMockCSIDriver(driverOpts CSIMockDriverOpts) MockCSITestDriver {
 				storageframework.CapMultiplePVsSameID: true,
 			},
 		},
-		manifests:              driverManifests,
-		podInfo:                driverOpts.PodInfo,
-		storageCapacity:        driverOpts.StorageCapacity,
-		enableTopology:         driverOpts.EnableTopology,
-		attachable:             !driverOpts.DisableAttach,
-		attachLimit:            driverOpts.AttachLimit,
-		enableNodeExpansion:    driverOpts.EnableNodeExpansion,
-		tokenRequests:          driverOpts.TokenRequests,
-		requiresRepublish:      driverOpts.RequiresRepublish,
-		fsGroupPolicy:          driverOpts.FSGroupPolicy,
-		enableVolumeMountGroup: driverOpts.EnableVolumeMountGroup,
-		enableSELinuxMount:     driverOpts.EnableSELinuxMount,
-		embedded:               driverOpts.Embedded,
-		hooks:                  driverOpts.Hooks,
+		manifests:                     driverManifests,
+		podInfo:                       driverOpts.PodInfo,
+		storageCapacity:               driverOpts.StorageCapacity,
+		enableTopology:                driverOpts.EnableTopology,
+		attachable:                    !driverOpts.DisableAttach,
+		attachLimit:                   driverOpts.AttachLimit,
+		enableNodeExpansion:           driverOpts.EnableNodeExpansion,
+		tokenRequests:                 driverOpts.TokenRequests,
+		requiresRepublish:             driverOpts.RequiresRepublish,
+		fsGroupPolicy:                 driverOpts.FSGroupPolicy,
+		enableVolumeMountGroup:        driverOpts.EnableVolumeMountGroup,
+		enableSELinuxMount:            driverOpts.EnableSELinuxMount,
+		enableRecoverExpansionFailure: driverOpts.EnableRecoverExpansionFailure,
+		embedded:                      driverOpts.Embedded,
+		hooks:                         driverOpts.Hooks,
 	}
 }
 
@@ -522,7 +526,7 @@ func (m *mockCSIDriver) GetDriverInfo() *storageframework.DriverInfo {
 func (m *mockCSIDriver) SkipUnsupportedTest(pattern storageframework.TestPattern) {
 }
 
-func (m *mockCSIDriver) GetDynamicProvisionStorageClass(config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
+func (m *mockCSIDriver) GetDynamicProvisionStorageClass(ctx context.Context, config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
 	provisioner := config.GetUniqueDriverName()
 	parameters := map[string]string{}
 	ns := config.Framework.Namespace.Name
@@ -530,18 +534,18 @@ func (m *mockCSIDriver) GetDynamicProvisionStorageClass(config *storageframework
 	return storageframework.GetStorageClass(provisioner, parameters, nil, ns)
 }
 
-func (m *mockCSIDriver) GetSnapshotClass(config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
+func (m *mockCSIDriver) GetSnapshotClass(ctx context.Context, config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
 	snapshotter := m.driverInfo.Name + "-" + config.Framework.UniqueName
 	ns := config.Framework.Namespace.Name
 
 	return utils.GenerateSnapshotClassSpec(snapshotter, parameters, ns)
 }
 
-func (m *mockCSIDriver) PrepareTest(f *framework.Framework) *storageframework.PerTestConfig {
+func (m *mockCSIDriver) PrepareTest(ctx context.Context, f *framework.Framework) *storageframework.PerTestConfig {
 	m.clientSet = f.ClientSet
 
 	// Create secondary namespace which will be used for creating driver
-	m.driverNamespace = utils.CreateDriverNamespace(f)
+	m.driverNamespace = utils.CreateDriverNamespace(ctx, f)
 	driverns := m.driverNamespace.Name
 	testns := f.Namespace.Name
 
@@ -550,11 +554,11 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) *storageframework.Pe
 	} else {
 		ginkgo.By("deploying csi mock driver")
 	}
-	cancelLogging := utils.StartPodLogs(f, m.driverNamespace)
+	cancelLogging := utils.StartPodLogs(ctx, f, m.driverNamespace)
 	cs := f.ClientSet
 
 	// pods should be scheduled on the node
-	node, err := e2enode.GetRandomReadySchedulableNode(cs)
+	node, err := e2enode.GetRandomReadySchedulableNode(ctx, cs)
 	framework.ExpectNoError(err)
 
 	embeddedCleanup := func() {}
@@ -572,6 +576,9 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) *storageframework.Pe
 		// this process.
 		podname := "csi-mockplugin-0"
 		containername := "mock"
+
+		// Must keep running even after the test context is cancelled
+		// for cleanup callbacks.
 		ctx, cancel := context.WithCancel(context.Background())
 		serviceConfig := mockservice.Config{
 			DisableAttach:            !m.attachable,
@@ -594,7 +601,6 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) *storageframework.Pe
 			Node:       s,
 		}
 		m.embeddedCSIDriver = mockdriver.NewCSIDriver(servers)
-
 		l, err := proxy.Listen(ctx, f.ClientSet, f.ClientConfig(),
 			proxy.Addr{
 				Namespace:     m.driverNamespace.Name,
@@ -603,6 +609,7 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) *storageframework.Pe
 				Port:          9000,
 			},
 		)
+
 		framework.ExpectNoError(err, "start connecting to proxy pod")
 		err = m.embeddedCSIDriver.Start(l, m.interceptGRPC)
 		framework.ExpectNoError(err, "start mock driver")
@@ -661,8 +668,13 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) *storageframework.Pe
 		RequiresRepublish: m.requiresRepublish,
 		FSGroupPolicy:     m.fsGroupPolicy,
 		SELinuxMount:      m.enableSELinuxMount,
+		Features:          map[string][]string{},
 	}
-	cleanup, err := utils.CreateFromManifests(f, m.driverNamespace, func(item interface{}) error {
+
+	if m.enableRecoverExpansionFailure {
+		o.Features["csi-resizer"] = []string{"RecoverVolumeExpansionFailure=true"}
+	}
+	err = utils.CreateFromManifests(ctx, f, m.driverNamespace, func(item interface{}) error {
 		if err := utils.PatchCSIDeployment(config.Framework, o, item); err != nil {
 			return err
 		}
@@ -693,12 +705,11 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) *storageframework.Pe
 		"mock",
 		testns,
 		driverns,
-		cleanup,
 		cancelLogging)
 
-	ginkgo.DeferCleanup(func() {
+	ginkgo.DeferCleanup(func(ctx context.Context) {
 		embeddedCleanup()
-		driverCleanupFunc()
+		driverCleanupFunc(ctx)
 	})
 
 	return config
@@ -725,7 +736,7 @@ func (m *mockCSIDriver) interceptGRPC(ctx context.Context, req interface{}, info
 	return
 }
 
-func (m *mockCSIDriver) GetCalls() ([]MockCSICall, error) {
+func (m *mockCSIDriver) GetCalls(ctx context.Context) ([]MockCSICall, error) {
 	if m.embedded {
 		return m.calls.Get(), nil
 	}
@@ -740,9 +751,9 @@ func (m *mockCSIDriver) GetCalls() ([]MockCSICall, error) {
 	driverContainerName := "mock"
 
 	// Load logs of driver pod
-	log, err := e2epod.GetPodLogs(m.clientSet, m.driverNamespace.Name, driverPodName, driverContainerName)
+	log, err := e2epod.GetPodLogs(ctx, m.clientSet, m.driverNamespace.Name, driverPodName, driverContainerName)
 	if err != nil {
-		return nil, fmt.Errorf("could not load CSI driver logs: %s", err)
+		return nil, fmt.Errorf("could not load CSI driver logs: %w", err)
 	}
 
 	logLines := strings.Split(log, "\n")
@@ -806,15 +817,16 @@ func InitGcePDCSIDriver() storageframework.TestDriver {
 				storageframework.CapMultiPODs:   true,
 				// GCE supports volume limits, but the test creates large
 				// number of volumes and times out test suites.
-				storageframework.CapVolumeLimits:        false,
-				storageframework.CapTopology:            true,
-				storageframework.CapControllerExpansion: true,
-				storageframework.CapOfflineExpansion:    true,
-				storageframework.CapOnlineExpansion:     true,
-				storageframework.CapNodeExpansion:       true,
-				storageframework.CapSnapshotDataSource:  true,
-				storageframework.CapReadWriteOncePod:    true,
-				storageframework.CapMultiplePVsSameID:   true,
+				storageframework.CapVolumeLimits:                   false,
+				storageframework.CapTopology:                       true,
+				storageframework.CapControllerExpansion:            true,
+				storageframework.CapOfflineExpansion:               true,
+				storageframework.CapOnlineExpansion:                true,
+				storageframework.CapNodeExpansion:                  true,
+				storageframework.CapSnapshotDataSource:             true,
+				storageframework.CapReadWriteOncePod:               true,
+				storageframework.CapMultiplePVsSameID:              true,
+				storageframework.CapFSResizeFromSourceNotSupported: true, //TODO: remove when CI tests use the fixed driver with: https://github.com/kubernetes-sigs/gcp-compute-persistent-disk-csi-driver/pull/972
 			},
 			RequiredAccessModes: []v1.PersistentVolumeAccessMode{v1.ReadWriteOnce},
 			TopologyKeys:        []string{GCEPDCSIZoneTopologyKey},
@@ -848,7 +860,7 @@ func (g *gcePDCSIDriver) SkipUnsupportedTest(pattern storageframework.TestPatter
 	}
 }
 
-func (g *gcePDCSIDriver) GetDynamicProvisionStorageClass(config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
+func (g *gcePDCSIDriver) GetDynamicProvisionStorageClass(ctx context.Context, config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
 	ns := config.Framework.Namespace.Name
 	provisioner := g.driverInfo.Name
 
@@ -861,14 +873,14 @@ func (g *gcePDCSIDriver) GetDynamicProvisionStorageClass(config *storageframewor
 	return storageframework.GetStorageClass(provisioner, parameters, &delayedBinding, ns)
 }
 
-func (g *gcePDCSIDriver) GetSnapshotClass(config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
+func (g *gcePDCSIDriver) GetSnapshotClass(ctx context.Context, config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
 	snapshotter := g.driverInfo.Name
 	ns := config.Framework.Namespace.Name
 
 	return utils.GenerateSnapshotClassSpec(snapshotter, parameters, ns)
 }
 
-func (g *gcePDCSIDriver) PrepareTest(f *framework.Framework) *storageframework.PerTestConfig {
+func (g *gcePDCSIDriver) PrepareTest(ctx context.Context, f *framework.Framework) *storageframework.PerTestConfig {
 	testns := f.Namespace.Name
 	cfg := &storageframework.PerTestConfig{
 		Driver:    g,
@@ -883,10 +895,10 @@ func (g *gcePDCSIDriver) PrepareTest(f *framework.Framework) *storageframework.P
 
 	ginkgo.By("deploying csi gce-pd driver")
 	// Create secondary namespace which will be used for creating driver
-	driverNamespace := utils.CreateDriverNamespace(f)
+	driverNamespace := utils.CreateDriverNamespace(ctx, f)
 	driverns := driverNamespace.Name
 
-	cancelLogging := utils.StartPodLogs(f, driverNamespace)
+	cancelLogging := utils.StartPodLogs(ctx, f, driverNamespace)
 	// It would be safer to rename the gcePD driver, but that
 	// hasn't been done before either and attempts to do so now led to
 	// errors during driver registration, therefore it is disabled
@@ -909,12 +921,12 @@ func (g *gcePDCSIDriver) PrepareTest(f *framework.Framework) *storageframework.P
 		"test/e2e/testing-manifests/storage-csi/gce-pd/controller_ss.yaml",
 	}
 
-	cleanup, err := utils.CreateFromManifests(f, driverNamespace, nil, manifests...)
+	err := utils.CreateFromManifests(ctx, f, driverNamespace, nil, manifests...)
 	if err != nil {
 		framework.Failf("deploying csi gce-pd driver: %v", err)
 	}
 
-	if err = WaitForCSIDriverRegistrationOnAllNodes(GCEPDCSIDriverName, f.ClientSet); err != nil {
+	if err = WaitForCSIDriverRegistrationOnAllNodes(ctx, GCEPDCSIDriverName, f.ClientSet); err != nil {
 		framework.Failf("waiting for csi driver node registration on: %v", err)
 	}
 
@@ -923,7 +935,6 @@ func (g *gcePDCSIDriver) PrepareTest(f *framework.Framework) *storageframework.P
 		"gce-pd",
 		testns,
 		driverns,
-		cleanup,
 		cancelLogging)
 	ginkgo.DeferCleanup(cleanupFunc)
 
@@ -937,13 +948,13 @@ func (g *gcePDCSIDriver) PrepareTest(f *framework.Framework) *storageframework.P
 
 // WaitForCSIDriverRegistrationOnAllNodes waits for the CSINode object to be updated
 // with the given driver on all schedulable nodes.
-func WaitForCSIDriverRegistrationOnAllNodes(driverName string, cs clientset.Interface) error {
-	nodes, err := e2enode.GetReadySchedulableNodes(cs)
+func WaitForCSIDriverRegistrationOnAllNodes(ctx context.Context, driverName string, cs clientset.Interface) error {
+	nodes, err := e2enode.GetReadySchedulableNodes(ctx, cs)
 	if err != nil {
 		return err
 	}
 	for _, node := range nodes.Items {
-		if err := WaitForCSIDriverRegistrationOnNode(node.Name, driverName, cs); err != nil {
+		if err := WaitForCSIDriverRegistrationOnNode(ctx, node.Name, driverName, cs); err != nil {
 			return err
 		}
 	}
@@ -951,7 +962,7 @@ func WaitForCSIDriverRegistrationOnAllNodes(driverName string, cs clientset.Inte
 }
 
 // WaitForCSIDriverRegistrationOnNode waits for the CSINode object generated by the node-registrar on a certain node
-func WaitForCSIDriverRegistrationOnNode(nodeName string, driverName string, cs clientset.Interface) error {
+func WaitForCSIDriverRegistrationOnNode(ctx context.Context, nodeName string, driverName string, cs clientset.Interface) error {
 	framework.Logf("waiting for CSIDriver %v to register on node %v", driverName, nodeName)
 
 	// About 8.6 minutes timeout
@@ -962,7 +973,7 @@ func WaitForCSIDriverRegistrationOnNode(nodeName string, driverName string, cs c
 	}
 
 	waitErr := wait.ExponentialBackoff(backoff, func() (bool, error) {
-		csiNode, err := cs.StorageV1().CSINodes().Get(context.TODO(), nodeName, metav1.GetOptions{})
+		csiNode, err := cs.StorageV1().CSINodes().Get(ctx, nodeName, metav1.GetOptions{})
 		if err != nil && !apierrors.IsNotFound(err) {
 			return false, err
 		}
@@ -996,22 +1007,21 @@ func tryFunc(f func()) error {
 func generateDriverCleanupFunc(
 	f *framework.Framework,
 	driverName, testns, driverns string,
-	driverCleanup, cancelLogging func()) func() {
+	cancelLogging func()) func(ctx context.Context) {
 
 	// Cleanup CSI driver and namespaces. This function needs to be idempotent and can be
 	// concurrently called from defer (or AfterEach) and AfterSuite action hooks.
-	cleanupFunc := func() {
+	cleanupFunc := func(ctx context.Context) {
 		ginkgo.By(fmt.Sprintf("deleting the test namespace: %s", testns))
 		// Delete the primary namespace but it's okay to fail here because this namespace will
 		// also be deleted by framework.Aftereach hook
-		tryFunc(func() { f.DeleteNamespace(testns) })
+		_ = tryFunc(func() { f.DeleteNamespace(ctx, testns) })
 
 		ginkgo.By(fmt.Sprintf("uninstalling csi %s driver", driverName))
-		tryFunc(driverCleanup)
-		tryFunc(cancelLogging)
+		_ = tryFunc(cancelLogging)
 
 		ginkgo.By(fmt.Sprintf("deleting the driver namespace: %s", driverns))
-		tryFunc(func() { f.DeleteNamespace(driverns) })
+		_ = tryFunc(func() { f.DeleteNamespace(ctx, driverns) })
 	}
 
 	return cleanupFunc
