@@ -23,12 +23,13 @@ import (
 	"errors"
 	"fmt"
 
+	"github.com/onsi/ginkgo/v2"
+
 	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
 	storagev1 "k8s.io/api/storage/v1"
 	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
-	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -49,7 +50,7 @@ import (
 //
 // LoadFromManifests has some limitations:
 //   - aliases are not supported (i.e. use serviceAccountName instead of the deprecated serviceAccount,
-//     https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.11/#podspec-v1-core)
+//     https://kubernetes.io/docs/reference/kubernetes-api/workload-resources/pod-v1)
 //     and silently ignored
 //   - the latest stable API version for each item is used, regardless of what
 //     is specified in the manifest files
@@ -140,21 +141,7 @@ func PatchItems(f *framework.Framework, driverNamespace *v1.Namespace, items ...
 // PatchItems has the some limitations as LoadFromManifests:
 // - only some common items are supported, unknown ones trigger an error
 // - only the latest stable API version for each item is supported
-func CreateItems(f *framework.Framework, ns *v1.Namespace, items ...interface{}) (func(), error) {
-	var destructors []func() error
-	cleanup := func() {
-		// TODO (?): use same logic as framework.go for determining
-		// whether we are expected to clean up? This would change the
-		// meaning of the -delete-namespace and -delete-namespace-on-failure
-		// command line flags, because they would also start to apply
-		// to non-namespaced items.
-		for _, destructor := range destructors {
-			if err := destructor(); err != nil && !apierrors.IsNotFound(err) {
-				framework.Logf("deleting failed: %s", err)
-			}
-		}
-	}
-
+func CreateItems(ctx context.Context, f *framework.Framework, ns *v1.Namespace, items ...interface{}) error {
 	var result error
 	for _, item := range items {
 		// Each factory knows which item(s) it supports, so try each one.
@@ -164,12 +151,9 @@ func CreateItems(f *framework.Framework, ns *v1.Namespace, items ...interface{})
 		// description = fmt.Sprintf("%s:\n%s", description, PrettyPrint(item))
 		framework.Logf("creating %s", description)
 		for _, factory := range factories {
-			destructor, err := factory.Create(f, ns, item)
+			destructor, err := factory.Create(ctx, f, ns, item)
 			if destructor != nil {
-				destructors = append(destructors, func() error {
-					framework.Logf("deleting %s", description)
-					return destructor()
-				})
+				ginkgo.DeferCleanup(framework.IgnoreNotFound(destructor), framework.AnnotatedLocation(fmt.Sprintf("deleting %s", description)))
 			}
 			if err == nil {
 				done = true
@@ -185,33 +169,28 @@ func CreateItems(f *framework.Framework, ns *v1.Namespace, items ...interface{})
 		}
 	}
 
-	if result != nil {
-		cleanup()
-		return nil, result
-	}
-
-	return cleanup, nil
+	return result
 }
 
 // CreateFromManifests is a combination of LoadFromManifests,
 // PatchItems, patching with an optional custom function,
 // and CreateItems.
-func CreateFromManifests(f *framework.Framework, driverNamespace *v1.Namespace, patch func(item interface{}) error, files ...string) (func(), error) {
+func CreateFromManifests(ctx context.Context, f *framework.Framework, driverNamespace *v1.Namespace, patch func(item interface{}) error, files ...string) error {
 	items, err := LoadFromManifests(files...)
 	if err != nil {
-		return nil, fmt.Errorf("CreateFromManifests: %w", err)
+		return fmt.Errorf("CreateFromManifests: %w", err)
 	}
 	if err := PatchItems(f, driverNamespace, items...); err != nil {
-		return nil, err
+		return err
 	}
 	if patch != nil {
 		for _, item := range items {
 			if err := patch(item); err != nil {
-				return nil, err
+				return err
 			}
 		}
 	}
-	return CreateItems(f, driverNamespace, items...)
+	return CreateItems(ctx, f, driverNamespace, items...)
 }
 
 // What is a subset of metav1.TypeMeta which (in contrast to
@@ -251,7 +230,7 @@ type ItemFactory interface {
 	// error or a cleanup function for the created item.
 	// If the item is of an unsupported type, it must return
 	// an error that has errorItemNotSupported as cause.
-	Create(f *framework.Framework, ns *v1.Namespace, item interface{}) (func() error, error)
+	Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, item interface{}) (func(ctx context.Context) error, error)
 }
 
 // describeItem always returns a string that describes the item,
@@ -410,17 +389,17 @@ func (f *serviceAccountFactory) New() runtime.Object {
 	return &v1.ServiceAccount{}
 }
 
-func (*serviceAccountFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*serviceAccountFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*v1.ServiceAccount)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 	client := f.ClientSet.CoreV1().ServiceAccounts(ns.Name)
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create ServiceAccount: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -430,7 +409,7 @@ func (f *clusterRoleFactory) New() runtime.Object {
 	return &rbacv1.ClusterRole{}
 }
 
-func (*clusterRoleFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*clusterRoleFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*rbacv1.ClusterRole)
 	if !ok {
 		return nil, errorItemNotSupported
@@ -438,11 +417,11 @@ func (*clusterRoleFactory) Create(f *framework.Framework, ns *v1.Namespace, i in
 
 	framework.Logf("Define cluster role %v", item.GetName())
 	client := f.ClientSet.RbacV1().ClusterRoles()
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create ClusterRole: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -452,18 +431,18 @@ func (f *clusterRoleBindingFactory) New() runtime.Object {
 	return &rbacv1.ClusterRoleBinding{}
 }
 
-func (*clusterRoleBindingFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*clusterRoleBindingFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*rbacv1.ClusterRoleBinding)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.RbacV1().ClusterRoleBindings()
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create ClusterRoleBinding: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -473,18 +452,18 @@ func (f *roleFactory) New() runtime.Object {
 	return &rbacv1.Role{}
 }
 
-func (*roleFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*roleFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*rbacv1.Role)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.RbacV1().Roles(ns.Name)
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create Role: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -494,18 +473,18 @@ func (f *roleBindingFactory) New() runtime.Object {
 	return &rbacv1.RoleBinding{}
 }
 
-func (*roleBindingFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*roleBindingFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*rbacv1.RoleBinding)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.RbacV1().RoleBindings(ns.Name)
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create RoleBinding: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -515,18 +494,18 @@ func (f *serviceFactory) New() runtime.Object {
 	return &v1.Service{}
 }
 
-func (*serviceFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*serviceFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*v1.Service)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.CoreV1().Services(ns.Name)
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create Service: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -536,18 +515,18 @@ func (f *statefulSetFactory) New() runtime.Object {
 	return &appsv1.StatefulSet{}
 }
 
-func (*statefulSetFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*statefulSetFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*appsv1.StatefulSet)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.AppsV1().StatefulSets(ns.Name)
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create StatefulSet: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -557,18 +536,18 @@ func (f *deploymentFactory) New() runtime.Object {
 	return &appsv1.Deployment{}
 }
 
-func (*deploymentFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*deploymentFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*appsv1.Deployment)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.AppsV1().Deployments(ns.Name)
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create Deployment: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -578,18 +557,18 @@ func (f *daemonSetFactory) New() runtime.Object {
 	return &appsv1.DaemonSet{}
 }
 
-func (*daemonSetFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*daemonSetFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*appsv1.DaemonSet)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.AppsV1().DaemonSets(ns.Name)
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create DaemonSet: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -599,18 +578,18 @@ func (f *replicaSetFactory) New() runtime.Object {
 	return &appsv1.ReplicaSet{}
 }
 
-func (*replicaSetFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*replicaSetFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*appsv1.ReplicaSet)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.AppsV1().ReplicaSets(ns.Name)
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create ReplicaSet: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -620,18 +599,18 @@ func (f *storageClassFactory) New() runtime.Object {
 	return &storagev1.StorageClass{}
 }
 
-func (*storageClassFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*storageClassFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*storagev1.StorageClass)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.StorageV1().StorageClasses()
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create StorageClass: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -641,18 +620,18 @@ func (f *csiDriverFactory) New() runtime.Object {
 	return &storagev1.CSIDriver{}
 }
 
-func (*csiDriverFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*csiDriverFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*storagev1.CSIDriver)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.StorageV1().CSIDrivers()
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create CSIDriver: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -662,18 +641,18 @@ func (f *secretFactory) New() runtime.Object {
 	return &v1.Secret{}
 }
 
-func (*secretFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*secretFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	item, ok := i.(*v1.Secret)
 	if !ok {
 		return nil, errorItemNotSupported
 	}
 
 	client := f.ClientSet.CoreV1().Secrets(ns.Name)
-	if _, err := client.Create(context.TODO(), item, metav1.CreateOptions{}); err != nil {
+	if _, err := client.Create(ctx, item, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create Secret: %w", err)
 	}
-	return func() error {
-		return client.Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return client.Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
@@ -683,7 +662,7 @@ func (f *customResourceDefinitionFactory) New() runtime.Object {
 	return &apiextensionsv1.CustomResourceDefinition{}
 }
 
-func (*customResourceDefinitionFactory) Create(f *framework.Framework, ns *v1.Namespace, i interface{}) (func() error, error) {
+func (*customResourceDefinitionFactory) Create(ctx context.Context, f *framework.Framework, ns *v1.Namespace, i interface{}) (func(ctx context.Context) error, error) {
 	var err error
 	unstructCRD := &unstructured.Unstructured{}
 	gvr := schema.GroupVersionResource{Group: "apiextensions.k8s.io", Version: "v1", Resource: "customresourcedefinitions"}
@@ -698,11 +677,11 @@ func (*customResourceDefinitionFactory) Create(f *framework.Framework, ns *v1.Na
 		return nil, err
 	}
 
-	if _, err = f.DynamicClient.Resource(gvr).Create(context.TODO(), unstructCRD, metav1.CreateOptions{}); err != nil {
+	if _, err = f.DynamicClient.Resource(gvr).Create(ctx, unstructCRD, metav1.CreateOptions{}); err != nil {
 		return nil, fmt.Errorf("create CustomResourceDefinition: %w", err)
 	}
-	return func() error {
-		return f.DynamicClient.Resource(gvr).Delete(context.TODO(), item.GetName(), metav1.DeleteOptions{})
+	return func(ctx context.Context) error {
+		return f.DynamicClient.Resource(gvr).Delete(ctx, item.GetName(), metav1.DeleteOptions{})
 	}, nil
 }
 
