@@ -55,7 +55,7 @@ func getOOMScoreForPid(pid int) (int, error) {
 func validateOOMScoreAdjSetting(pid int, expectedOOMScoreAdj int) error {
 	oomScore, err := getOOMScoreForPid(pid)
 	if err != nil {
-		return fmt.Errorf("failed to get oom_score_adj for %d: %v", pid, err)
+		return fmt.Errorf("failed to get oom_score_adj for %d: %w", pid, err)
 	}
 	if expectedOOMScoreAdj != oomScore {
 		return fmt.Errorf("expected pid %d's oom_score_adj to be %d; found %d", pid, expectedOOMScoreAdj, oomScore)
@@ -82,25 +82,25 @@ var _ = SIGDescribe("Container Manager Misc [Serial]", func() {
 	f.NamespacePodSecurityEnforceLevel = admissionapi.LevelPrivileged
 	ginkgo.Describe("Validate OOM score adjustments [NodeFeature:OOMScoreAdj]", func() {
 		ginkgo.Context("once the node is setup", func() {
-			ginkgo.It("container runtime's oom-score-adj should be -999", func() {
+			ginkgo.It("container runtime's oom-score-adj should be -999", func(ctx context.Context) {
 				runtimePids, err := getPidsForProcess(framework.TestContext.ContainerRuntimeProcessName, framework.TestContext.ContainerRuntimePidFile)
 				framework.ExpectNoError(err, "failed to get list of container runtime pids")
 				for _, pid := range runtimePids {
-					gomega.Eventually(func() error {
+					gomega.Eventually(ctx, func() error {
 						return validateOOMScoreAdjSetting(pid, -999)
 					}, 5*time.Minute, 30*time.Second).Should(gomega.BeNil())
 				}
 			})
-			ginkgo.It("Kubelet's oom-score-adj should be -999", func() {
+			ginkgo.It("Kubelet's oom-score-adj should be -999", func(ctx context.Context) {
 				kubeletPids, err := getPidsForProcess(kubeletProcessName, "")
 				framework.ExpectNoError(err, "failed to get list of kubelet pids")
 				framework.ExpectEqual(len(kubeletPids), 1, "expected only one kubelet process; found %d", len(kubeletPids))
-				gomega.Eventually(func() error {
+				gomega.Eventually(ctx, func() error {
 					return validateOOMScoreAdjSetting(kubeletPids[0], -999)
 				}, 5*time.Minute, 30*time.Second).Should(gomega.BeNil())
 			})
 			ginkgo.Context("", func() {
-				ginkgo.It("pod infra containers oom-score-adj should be -998 and best effort container's should be 1000", func() {
+				ginkgo.It("pod infra containers oom-score-adj should be -998 and best effort container's should be 1000", func(ctx context.Context) {
 					// Take a snapshot of existing pause processes. These were
 					// created before this test, and may not be infra
 					// containers. They should be excluded from the test.
@@ -110,7 +110,7 @@ var _ = SIGDescribe("Container Manager Misc [Serial]", func() {
 
 					podClient := e2epod.NewPodClient(f)
 					podName := "besteffort" + string(uuid.NewUUID())
-					podClient.Create(&v1.Pod{
+					podClient.Create(ctx, &v1.Pod{
 						ObjectMeta: metav1.ObjectMeta{
 							Name: podName,
 						},
@@ -126,10 +126,10 @@ var _ = SIGDescribe("Container Manager Misc [Serial]", func() {
 
 					var pausePids []int
 					ginkgo.By("checking infra container's oom-score-adj")
-					gomega.Eventually(func() error {
+					gomega.Eventually(ctx, func() error {
 						pausePids, err = getPidsForProcess("pause", "")
 						if err != nil {
-							return fmt.Errorf("failed to get list of pause pids: %v", err)
+							return fmt.Errorf("failed to get list of pause pids: %w", err)
 						}
 						for _, pid := range pausePids {
 							if existingPausePIDSet.Has(pid) {
@@ -144,10 +144,10 @@ var _ = SIGDescribe("Container Manager Misc [Serial]", func() {
 					}, 2*time.Minute, time.Second*4).Should(gomega.BeNil())
 					var shPids []int
 					ginkgo.By("checking besteffort container's oom-score-adj")
-					gomega.Eventually(func() error {
+					gomega.Eventually(ctx, func() error {
 						shPids, err = getPidsForProcess("agnhost", "")
 						if err != nil {
-							return fmt.Errorf("failed to get list of serve hostname process pids: %v", err)
+							return fmt.Errorf("failed to get list of serve hostname process pids: %w", err)
 						}
 						if len(shPids) != 1 {
 							return fmt.Errorf("expected only one agnhost process; found %d", len(shPids))
@@ -174,10 +174,10 @@ var _ = SIGDescribe("Container Manager Misc [Serial]", func() {
 					}
 				})
 			})
-			ginkgo.It("guaranteed container's oom-score-adj should be -998", func() {
+			ginkgo.It("guaranteed container's oom-score-adj should be -998", func(ctx context.Context) {
 				podClient := e2epod.NewPodClient(f)
 				podName := "guaranteed" + string(uuid.NewUUID())
-				podClient.Create(&v1.Pod{
+				podClient.Create(ctx, &v1.Pod{
 					ObjectMeta: metav1.ObjectMeta{
 						Name: podName,
 					},
@@ -200,10 +200,10 @@ var _ = SIGDescribe("Container Manager Misc [Serial]", func() {
 					ngPids []int
 					err    error
 				)
-				gomega.Eventually(func() error {
+				gomega.Eventually(ctx, func() error {
 					ngPids, err = getPidsForProcess("nginx", "")
 					if err != nil {
-						return fmt.Errorf("failed to get list of nginx process pids: %v", err)
+						return fmt.Errorf("failed to get list of nginx process pids: %w", err)
 					}
 					for _, pid := range ngPids {
 						if err := validateOOMScoreAdjSetting(pid, -998); err != nil {
@@ -215,10 +215,10 @@ var _ = SIGDescribe("Container Manager Misc [Serial]", func() {
 				}, 2*time.Minute, time.Second*4).Should(gomega.BeNil())
 
 			})
-			ginkgo.It("burstable container's oom-score-adj should be between [2, 1000)", func() {
+			ginkgo.It("burstable container's oom-score-adj should be between [2, 1000)", func(ctx context.Context) {
 				podClient := e2epod.NewPodClient(f)
 				podName := "burstable" + string(uuid.NewUUID())
-				podClient.Create(&v1.Pod{
+				podClient.Create(ctx, &v1.Pod{
 					ObjectMeta: metav1.ObjectMeta{
 						Name: podName,
 					},
@@ -242,10 +242,10 @@ var _ = SIGDescribe("Container Manager Misc [Serial]", func() {
 					wsPids []int
 					err    error
 				)
-				gomega.Eventually(func() error {
+				gomega.Eventually(ctx, func() error {
 					wsPids, err = getPidsForProcess("agnhost", "")
 					if err != nil {
-						return fmt.Errorf("failed to get list of test-webserver process pids: %v", err)
+						return fmt.Errorf("failed to get list of test-webserver process pids: %w", err)
 					}
 					for _, pid := range wsPids {
 						if err := validateOOMScoreAdjSettingIsInRange(pid, 2, 1000); err != nil {
