@@ -45,6 +45,7 @@ import (
 	v1 "k8s.io/api/core/v1"
 	schedulingv1 "k8s.io/api/scheduling/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/util/uuid"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/kubernetes/pkg/features"
 	kubeletconfig "k8s.io/kubernetes/pkg/kubelet/apis/config"
@@ -65,7 +66,7 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			nodeShutdownGracePeriod = 30 * time.Second
 		)
 
-		tempSetCurrentKubeletConfig(f, func(initialConfig *kubeletconfig.KubeletConfiguration) {
+		tempSetCurrentKubeletConfig(f, func(ctx context.Context, initialConfig *kubeletconfig.KubeletConfiguration) {
 			initialConfig.FeatureGates = map[string]bool{
 				string(features.GracefulNodeShutdown):                   true,
 				string(features.PodDisruptionConditions):                true,
@@ -74,9 +75,9 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			initialConfig.ShutdownGracePeriod = metav1.Duration{Duration: nodeShutdownGracePeriod}
 		})
 
-		ginkgo.BeforeEach(func() {
+		ginkgo.BeforeEach(func(ctx context.Context) {
 			ginkgo.By("Wait for the node to be ready")
-			waitForNodeReady()
+			waitForNodeReady(ctx)
 		})
 
 		ginkgo.AfterEach(func() {
@@ -85,22 +86,22 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			framework.ExpectNoError(err)
 		})
 
-		ginkgo.It("should add the DisruptionTarget pod failure condition to the evicted pods", func() {
-			nodeName := getNodeName(f)
+		ginkgo.It("should add the DisruptionTarget pod failure condition to the evicted pods", func(ctx context.Context) {
+			nodeName := getNodeName(ctx, f)
 			nodeSelector := fields.Set{
 				"spec.nodeName": nodeName,
 			}.AsSelector().String()
 
 			// Define test pods
 			pods := []*v1.Pod{
-				getGracePeriodOverrideTestPod("pod-to-evict", nodeName, 5, ""),
+				getGracePeriodOverrideTestPod("pod-to-evict-"+string(uuid.NewUUID()), nodeName, 5, ""),
 			}
 
 			ctx, cancel := context.WithCancel(context.Background())
 			defer cancel()
 
 			ginkgo.By("reating batch pods")
-			e2epod.NewPodClient(f).CreateBatch(pods)
+			e2epod.NewPodClient(f).CreateBatch(ctx, pods)
 
 			list, err := e2epod.NewPodClient(f).List(ctx, metav1.ListOptions{
 				FieldSelector: nodeSelector,
@@ -118,13 +119,13 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			framework.ExpectEqual(len(list.Items), len(pods), "the number of pods is not as expected")
 
 			for _, pod := range list.Items {
-				framework.Logf("Pod %q status conditions: %q", pod.Name, &pod.Status.Conditions)
+				framework.Logf("Pod (%v/%v) status conditions: %q", pod.Namespace, pod.Name, &pod.Status.Conditions)
 			}
 
 			ginkgo.By("Verifying batch pods are running")
 			for _, pod := range list.Items {
 				if podReady, err := testutils.PodRunningReady(&pod); err != nil || !podReady {
-					framework.Failf("Failed to start batch pod: %v", pod.Name)
+					framework.Failf("Failed to start batch pod: (%v/%v)", pod.Namespace, pod.Name)
 				}
 			}
 
@@ -145,12 +146,12 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 
 				for _, pod := range list.Items {
 					if !isPodShutdown(&pod) {
-						framework.Logf("Expecting pod to be shutdown, but it's not currently: Pod: %q, Pod Status %+v", pod.Name, pod.Status)
+						framework.Logf("Expecting pod to be shutdown, but it's not currently. Pod: (%v/%v), Pod Status Phase: %q, Pod Status Reason: %q", pod.Namespace, pod.Name, pod.Status.Phase, pod.Status.Reason)
 						return fmt.Errorf("pod should be shutdown, phase: %s", pod.Status.Phase)
 					}
 					podDisruptionCondition := e2epod.FindPodConditionByType(&pod.Status, v1.DisruptionTarget)
 					if podDisruptionCondition == nil {
-						framework.Failf("pod %q should have the condition: %q, pod status: %v", pod.Name, v1.DisruptionTarget, pod.Status)
+						framework.Failf("pod (%v/%v) should have the condition: %q, pod status: %v", pod.Namespace, pod.Name, v1.DisruptionTarget, pod.Status)
 					}
 				}
 				return nil
@@ -168,7 +169,7 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			nodeShutdownGracePeriodCriticalPods = 10 * time.Second
 		)
 
-		tempSetCurrentKubeletConfig(f, func(initialConfig *kubeletconfig.KubeletConfiguration) {
+		tempSetCurrentKubeletConfig(f, func(ctx context.Context, initialConfig *kubeletconfig.KubeletConfiguration) {
 			initialConfig.FeatureGates = map[string]bool{
 				string(features.GracefulNodeShutdown):                   true,
 				string(features.GracefulNodeShutdownBasedOnPodPriority): false,
@@ -177,47 +178,47 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			initialConfig.ShutdownGracePeriodCriticalPods = metav1.Duration{Duration: nodeShutdownGracePeriodCriticalPods}
 		})
 
-		ginkgo.BeforeEach(func() {
+		ginkgo.BeforeEach(func(ctx context.Context) {
 			ginkgo.By("Wait for the node to be ready")
-			waitForNodeReady()
+			waitForNodeReady(ctx)
 		})
 
-		ginkgo.AfterEach(func() {
+		ginkgo.AfterEach(func(ctx context.Context) {
 			ginkgo.By("Emitting Shutdown false signal; cancelling the shutdown")
 			err := emitSignalPrepareForShutdown(false)
 			framework.ExpectNoError(err)
 		})
 
-		ginkgo.It("should be able to gracefully shutdown pods with various grace periods", func() {
-			nodeName := getNodeName(f)
+		ginkgo.It("should be able to gracefully shutdown pods with various grace periods", func(ctx context.Context) {
+			nodeName := getNodeName(ctx, f)
 			nodeSelector := fields.Set{
 				"spec.nodeName": nodeName,
 			}.AsSelector().String()
 
 			// Define test pods
 			pods := []*v1.Pod{
-				getGracePeriodOverrideTestPod("period-120", nodeName, 120, ""),
-				getGracePeriodOverrideTestPod("period-5", nodeName, 5, ""),
-				getGracePeriodOverrideTestPod("period-critical-120", nodeName, 120, scheduling.SystemNodeCritical),
-				getGracePeriodOverrideTestPod("period-critical-5", nodeName, 5, scheduling.SystemNodeCritical),
+				getGracePeriodOverrideTestPod("period-120-"+string(uuid.NewUUID()), nodeName, 120, ""),
+				getGracePeriodOverrideTestPod("period-5-"+string(uuid.NewUUID()), nodeName, 5, ""),
+				getGracePeriodOverrideTestPod("period-critical-120-"+string(uuid.NewUUID()), nodeName, 120, scheduling.SystemNodeCritical),
+				getGracePeriodOverrideTestPod("period-critical-5-"+string(uuid.NewUUID()), nodeName, 5, scheduling.SystemNodeCritical),
 			}
 
 			ginkgo.By("Creating batch pods")
-			e2epod.NewPodClient(f).CreateBatch(pods)
+			e2epod.NewPodClient(f).CreateBatch(ctx, pods)
 
-			list, err := e2epod.NewPodClient(f).List(context.TODO(), metav1.ListOptions{
+			list, err := e2epod.NewPodClient(f).List(ctx, metav1.ListOptions{
 				FieldSelector: nodeSelector,
 			})
 			framework.ExpectNoError(err)
 			framework.ExpectEqual(len(list.Items), len(pods), "the number of pods is not as expected")
 
-			ctx, cancel := context.WithCancel(context.Background())
+			ctx, cancel := context.WithCancel(ctx)
 			defer cancel()
 			go func() {
 				defer ginkgo.GinkgoRecover()
 				w := &cache.ListWatch{
 					WatchFunc: func(options metav1.ListOptions) (watch.Interface, error) {
-						return f.ClientSet.CoreV1().Pods(f.Namespace.Name).Watch(context.TODO(), options)
+						return f.ClientSet.CoreV1().Pods(f.Namespace.Name).Watch(ctx, options)
 					},
 				}
 
@@ -252,8 +253,8 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 
 			ginkgo.By("Verifying that non-critical pods are shutdown")
 			// Not critical pod should be shutdown
-			gomega.Eventually(func() error {
-				list, err = e2epod.NewPodClient(f).List(context.TODO(), metav1.ListOptions{
+			gomega.Eventually(ctx, func(ctx context.Context) error {
+				list, err = e2epod.NewPodClient(f).List(ctx, metav1.ListOptions{
 					FieldSelector: nodeSelector,
 				})
 				if err != nil {
@@ -264,23 +265,23 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 				for _, pod := range list.Items {
 					if kubelettypes.IsCriticalPod(&pod) {
 						if isPodShutdown(&pod) {
-							framework.Logf("Expecting critical pod to be running, but it's not currently. Pod: %q, Pod Status %+v", pod.Name, pod.Status)
-							return fmt.Errorf("critical pod should not be shutdown, phase: %s", pod.Status.Phase)
+							framework.Logf("Expecting critical pod (%v/%v) to be running, but it's not currently. Pod Status %+v", pod.Namespace, pod.Name, pod.Status)
+							return fmt.Errorf("critical pod (%v/%v) should not be shutdown, phase: %s", pod.Namespace, pod.Name, pod.Status.Phase)
 						}
 					} else {
 						if !isPodShutdown(&pod) {
-							framework.Logf("Expecting non-critical pod to be shutdown, but it's not currently. Pod: %q, Pod Status %+v", pod.Name, pod.Status)
-							return fmt.Errorf("pod should be shutdown, phase: %s", pod.Status.Phase)
+							framework.Logf("Expecting non-critical pod (%v/%v) to be shutdown, but it's not currently. Pod Status %+v", pod.Name, pod.Status)
+							return fmt.Errorf("pod (%v/%v) should be shutdown, phase: %s", pod.Namespace, pod.Name, pod.Status.Phase)
 						}
 					}
 				}
 				return nil
-			}, podStatusUpdateTimeout, pollInterval).Should(gomega.BeNil())
+			}, podStatusUpdateTimeout, pollInterval).Should(gomega.Succeed())
 
 			ginkgo.By("Verifying that all pods are shutdown")
 			// All pod should be shutdown
-			gomega.Eventually(func() error {
-				list, err = e2epod.NewPodClient(f).List(context.TODO(), metav1.ListOptions{
+			gomega.Eventually(ctx, func(ctx context.Context) error {
+				list, err = e2epod.NewPodClient(f).List(ctx, metav1.ListOptions{
 					FieldSelector: nodeSelector,
 				})
 				if err != nil {
@@ -290,43 +291,43 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 
 				for _, pod := range list.Items {
 					if !isPodShutdown(&pod) {
-						framework.Logf("Expecting pod to be shutdown, but it's not currently: Pod: %q, Pod Status %+v", pod.Name, pod.Status)
-						return fmt.Errorf("pod should be shutdown, phase: %s", pod.Status.Phase)
+						framework.Logf("Expecting pod (%v/%v) to be shutdown, but it's not currently: Pod Status %+v", pod.Namespace, pod.Name, pod.Status)
+						return fmt.Errorf("pod (%v/%v) should be shutdown, phase: %s", pod.Namespace, pod.Name, pod.Status.Phase)
 					}
 				}
 				return nil
 			},
 				// Critical pod starts shutdown after (nodeShutdownGracePeriod-nodeShutdownGracePeriodCriticalPods)
 				podStatusUpdateTimeout+(nodeShutdownGracePeriod-nodeShutdownGracePeriodCriticalPods),
-				pollInterval).Should(gomega.BeNil())
+				pollInterval).Should(gomega.Succeed())
 
 		})
 
-		ginkgo.It("should be able to handle a cancelled shutdown", func() {
+		ginkgo.It("should be able to handle a cancelled shutdown", func(ctx context.Context) {
 			ginkgo.By("Emitting Shutdown signal")
 			err := emitSignalPrepareForShutdown(true)
 			framework.ExpectNoError(err)
-			gomega.Eventually(func() error {
-				isReady := getNodeReadyStatus(f)
+			gomega.Eventually(ctx, func(ctx context.Context) error {
+				isReady := getNodeReadyStatus(ctx, f)
 				if isReady {
 					return fmt.Errorf("node did not become shutdown as expected")
 				}
 				return nil
-			}, nodeStatusUpdateTimeout, pollInterval).Should(gomega.BeNil())
+			}, nodeStatusUpdateTimeout, pollInterval).Should(gomega.Succeed())
 
 			ginkgo.By("Emitting Shutdown false signal; cancelling the shutdown")
 			err = emitSignalPrepareForShutdown(false)
 			framework.ExpectNoError(err)
-			gomega.Eventually(func() error {
-				isReady := getNodeReadyStatus(f)
+			gomega.Eventually(ctx, func(ctx context.Context) error {
+				isReady := getNodeReadyStatus(ctx, f)
 				if !isReady {
 					return fmt.Errorf("node did not recover as expected")
 				}
 				return nil
-			}, nodeStatusUpdateTimeout, pollInterval).Should(gomega.BeNil())
+			}, nodeStatusUpdateTimeout, pollInterval).Should(gomega.Succeed())
 		})
 
-		ginkgo.It("after restart dbus, should be able to gracefully shutdown", func() {
+		ginkgo.It("after restart dbus, should be able to gracefully shutdown", func(ctx context.Context) {
 			// allows manual restart of dbus to work in Ubuntu.
 			err := overlayDbusConfig()
 			framework.ExpectNoError(err)
@@ -346,13 +347,13 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			err = emitSignalPrepareForShutdown(true)
 			framework.ExpectNoError(err)
 
-			gomega.Eventually(func() error {
-				isReady := getNodeReadyStatus(f)
+			gomega.Eventually(ctx, func(ctx context.Context) error {
+				isReady := getNodeReadyStatus(ctx, f)
 				if isReady {
 					return fmt.Errorf("node did not become shutdown as expected")
 				}
 				return nil
-			}, nodeStatusUpdateTimeout, pollInterval).Should(gomega.BeNil())
+			}, nodeStatusUpdateTimeout, pollInterval).Should(gomega.Succeed())
 		})
 	})
 
@@ -360,7 +361,7 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 
 		const (
 			pollInterval                 = 1 * time.Second
-			podStatusUpdateTimeout       = 10 * time.Second
+			podStatusUpdateTimeout       = 30 * time.Second
 			priorityClassesCreateTimeout = 10 * time.Second
 		)
 
@@ -370,7 +371,7 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			customClassC = getPriorityClass("custom-class-c", 1000)
 		)
 
-		tempSetCurrentKubeletConfig(f, func(initialConfig *kubeletconfig.KubeletConfiguration) {
+		tempSetCurrentKubeletConfig(f, func(ctx context.Context, initialConfig *kubeletconfig.KubeletConfiguration) {
 			initialConfig.FeatureGates = map[string]bool{
 				string(features.GracefulNodeShutdown):                   true,
 				string(features.GracefulNodeShutdownBasedOnPodPriority): true,
@@ -400,25 +401,25 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 
 		})
 
-		ginkgo.BeforeEach(func() {
+		ginkgo.BeforeEach(func(ctx context.Context) {
 			ginkgo.By("Wait for the node to be ready")
-			waitForNodeReady()
+			waitForNodeReady(ctx)
 			customClasses := []*schedulingv1.PriorityClass{customClassA, customClassB, customClassC}
 			for _, customClass := range customClasses {
-				_, err := f.ClientSet.SchedulingV1().PriorityClasses().Create(context.Background(), customClass, metav1.CreateOptions{})
+				_, err := f.ClientSet.SchedulingV1().PriorityClasses().Create(ctx, customClass, metav1.CreateOptions{})
 				if err != nil && !apierrors.IsAlreadyExists(err) {
 					framework.ExpectNoError(err)
 				}
 			}
-			gomega.Eventually(func() error {
+			gomega.Eventually(ctx, func(ctx context.Context) error {
 				for _, customClass := range customClasses {
-					_, err := f.ClientSet.SchedulingV1().PriorityClasses().Get(context.Background(), customClass.Name, metav1.GetOptions{})
+					_, err := f.ClientSet.SchedulingV1().PriorityClasses().Get(ctx, customClass.Name, metav1.GetOptions{})
 					if err != nil {
 						return err
 					}
 				}
 				return nil
-			}, priorityClassesCreateTimeout, pollInterval).Should(gomega.BeNil())
+			}, priorityClassesCreateTimeout, pollInterval).Should(gomega.Succeed())
 		})
 
 		ginkgo.AfterEach(func() {
@@ -427,54 +428,63 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			framework.ExpectNoError(err)
 		})
 
-		ginkgo.It("should be able to gracefully shutdown pods with various grace periods", func() {
-			nodeName := getNodeName(f)
+		ginkgo.It("should be able to gracefully shutdown pods with various grace periods", func(ctx context.Context) {
+			nodeName := getNodeName(ctx, f)
 			nodeSelector := fields.Set{
 				"spec.nodeName": nodeName,
 			}.AsSelector().String()
 
+			var (
+				period5Name         = "period-5-" + string(uuid.NewUUID())
+				periodC5Name        = "period-c-5-" + string(uuid.NewUUID())
+				periodB5Name        = "period-b-5-" + string(uuid.NewUUID())
+				periodA5Name        = "period-a-5-" + string(uuid.NewUUID())
+				periodCritical5Name = "period-critical-5-" + string(uuid.NewUUID())
+			)
+
 			// Define test pods
 			pods := []*v1.Pod{
-				getGracePeriodOverrideTestPod("period-5", nodeName, 5, ""),
-				getGracePeriodOverrideTestPod("period-c-5", nodeName, 5, customClassC.Name),
-				getGracePeriodOverrideTestPod("period-b-5", nodeName, 5, customClassB.Name),
-				getGracePeriodOverrideTestPod("period-a-5", nodeName, 5, customClassA.Name),
-				getGracePeriodOverrideTestPod("period-critical-5", nodeName, 5, scheduling.SystemNodeCritical),
+				getGracePeriodOverrideTestPod(period5Name, nodeName, 5, ""),
+				getGracePeriodOverrideTestPod(periodC5Name, nodeName, 5, customClassC.Name),
+				getGracePeriodOverrideTestPod(periodB5Name, nodeName, 5, customClassB.Name),
+				getGracePeriodOverrideTestPod(periodA5Name, nodeName, 5, customClassA.Name),
+				getGracePeriodOverrideTestPod(periodCritical5Name, nodeName, 5, scheduling.SystemNodeCritical),
 			}
 
 			// Expected down steps
 			downSteps := [][]string{
 				{
-					"period-5",
+					period5Name,
 				},
 				{
-					"period-5",
-					"period-c-5",
+					period5Name,
+					periodC5Name,
 				},
 				{
-					"period-5",
-					"period-c-5",
-					"period-b-5",
+
+					period5Name,
+					periodC5Name,
+					periodB5Name,
 				},
 				{
-					"period-5",
-					"period-c-5",
-					"period-b-5",
-					"period-a-5",
+					period5Name,
+					periodC5Name,
+					periodB5Name,
+					periodA5Name,
 				},
 				{
-					"period-5",
-					"period-c-5",
-					"period-b-5",
-					"period-a-5",
-					"period-critical-5",
+					period5Name,
+					periodC5Name,
+					periodB5Name,
+					periodA5Name,
+					periodCritical5Name,
 				},
 			}
 
 			ginkgo.By("Creating batch pods")
-			e2epod.NewPodClient(f).CreateBatch(pods)
+			e2epod.NewPodClient(f).CreateBatch(ctx, pods)
 
-			list, err := e2epod.NewPodClient(f).List(context.TODO(), metav1.ListOptions{
+			list, err := e2epod.NewPodClient(f).List(ctx, metav1.ListOptions{
 				FieldSelector: nodeSelector,
 			})
 			framework.ExpectNoError(err)
@@ -483,7 +493,7 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			ginkgo.By("Verifying batch pods are running")
 			for _, pod := range list.Items {
 				if podReady, err := testutils.PodRunningReady(&pod); err != nil || !podReady {
-					framework.Failf("Failed to start batch pod: %v", pod.Name)
+					framework.Failf("Failed to start batch pod: (%v/%v)", pod.Namespace, pod.Name)
 				}
 			}
 
@@ -494,8 +504,8 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 			ginkgo.By("Verifying that pods are shutdown")
 
 			for _, step := range downSteps {
-				gomega.Eventually(func() error {
-					list, err = e2epod.NewPodClient(f).List(context.TODO(), metav1.ListOptions{
+				gomega.Eventually(ctx, func(ctx context.Context) error {
+					list, err = e2epod.NewPodClient(f).List(ctx, metav1.ListOptions{
 						FieldSelector: nodeSelector,
 					})
 					if err != nil {
@@ -512,21 +522,21 @@ var _ = SIGDescribe("GracefulNodeShutdown [Serial] [NodeFeature:GracefulNodeShut
 						}
 						if !shouldShutdown {
 							if pod.Status.Phase != v1.PodRunning {
-								framework.Logf("Expecting pod to be running, but it's not currently. Pod: %q, Pod Status Phase: %q, Pod Status Reason: %q", pod.Name, pod.Status.Phase, pod.Status.Reason)
-								return fmt.Errorf("pod should not be shutdown, phase: %s, reason: %s", pod.Status.Phase, pod.Status.Reason)
+								framework.Logf("Expecting pod to be running, but it's not currently. Pod: (%v/%v), Pod Status Phase: %q, Pod Status Reason: %q", pod.Namespace, pod.Name, pod.Status.Phase, pod.Status.Reason)
+								return fmt.Errorf("pod (%v/%v) should not be shutdown, phase: %s, reason: %s", pod.Namespace, pod.Name, pod.Status.Phase, pod.Status.Reason)
 							}
 						} else {
 							if pod.Status.Reason != podShutdownReason {
-								framework.Logf("Expecting pod to be shutdown, but it's not currently. Pod: %q, Pod Status Phase: %q, Pod Status Reason: %q", pod.Name, pod.Status.Phase, pod.Status.Reason)
+								framework.Logf("Expecting pod to be shutdown, but it's not currently. Pod: (%v/%v), Pod Status Phase: %q, Pod Status Reason: %q", pod.Namespace, pod.Name, pod.Status.Phase, pod.Status.Reason)
 								for _, item := range list.Items {
 									framework.Logf("DEBUG %s, %s, %s", item.Name, item.Status.Phase, pod.Status.Reason)
 								}
-								return fmt.Errorf("pod should be shutdown, reason: %s", pod.Status.Reason)
+								return fmt.Errorf("pod (%v/%v) should be shutdown, reason: %s", pod.Namespace, pod.Name, pod.Status.Reason)
 							}
 						}
 					}
 					return nil
-				}, podStatusUpdateTimeout, pollInterval).Should(gomega.BeNil())
+				}, podStatusUpdateTimeout, pollInterval).Should(gomega.Succeed())
 			}
 
 			ginkgo.By("should have state file")
@@ -602,8 +612,8 @@ func emitSignalPrepareForShutdown(b bool) error {
 	return conn.Emit("/org/freedesktop/login1", "org.freedesktop.login1.Manager.PrepareForShutdown", b)
 }
 
-func getNodeReadyStatus(f *framework.Framework) bool {
-	nodeList, err := f.ClientSet.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{})
+func getNodeReadyStatus(ctx context.Context, f *framework.Framework) bool {
+	nodeList, err := f.ClientSet.CoreV1().Nodes().List(ctx, metav1.ListOptions{})
 	framework.ExpectNoError(err)
 	// Assuming that there is only one node, because this is a node e2e test.
 	framework.ExpectEqual(len(nodeList.Items), 1)
